# -*- coding: utf-8 -*-
"""Provide functions for the creation and manipulation of Rays.

A ray begins as a single point and extends
infinitely in a direction.

The first vector is the origin of the ray.
The second vector is the direction of the ray
relative to the origin.

The following functions will normalize the ray
direction to unit length.
Some functions may work correctly with directions
that are not unit length, but this may vary from
function to function.
"""
from __future__ import absolute_import, division, print_function
import numpy as np
from . import vector
from .utils import all_parameters_as_numpy_arrays, parameters_as_numpy_arrays


class index:
    #: The index of the start vector within the ray
    position = 0

    #: The index of the direction vector within the ray
    direction = 1


@parameters_as_numpy_arrays('start', 'direction')
def create(start, direction, dtype=None):
    dtype = dtype or start.dtype
    return np.array(
        [
            start,
            vector.normalize(direction)
        ],
        dtype=dtype
    )

@parameters_as_numpy_arrays('line')
def create_from_line(line, dtype=None):
    """Converts a line or line segment to a ray.
    """
    dtype = dtype or line.dtype
    # direction = vend - vstart
    return np.array(
        [
            line[0],
            vector.normalize(line[1] - line[0])
        ],
        dtype=dtype
    )

@all_parameters_as_numpy_arrays
def invert(r):
    r2 = r.copy()
    r2[1] *= -1
    return r2

@all_parameters_as_numpy_arrays
def position(ray):
    return ray[0].copy()

@all_parameters_as_numpy_arrays
def direction(ray):
    return ray[1].copy()




try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import rectangle


class test_rectangle(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.rectangle
        from pyrr import rectangle

    def test_create(self):
        result = rectangle.create()
        np.testing.assert_almost_equal(result, [[0,0],[1,1]], decimal=5)

    def test_create_dtype(self):
        result = rectangle.create(dtype=np.float)
        np.testing.assert_almost_equal(result, [[0.,0.],[1.,1.]], decimal=5)

    def test_create_zeros(self):
        result = rectangle.create_zeros()
        np.testing.assert_almost_equal(result, [[0,0],[0,0]], decimal=5)

    def test_create_from_bounds(self):
        result = rectangle.create_from_bounds(-1, 1, -2, 2)
        np.testing.assert_almost_equal(result, [[-1,-2],[2,4]], decimal=5)

    def test_bounds(self):
        rect = rectangle.create_from_bounds(-1, 1, -2, 2)
        result = rectangle.bounds(rect)
        np.testing.assert_almost_equal(result, (-1,1,-2,2), decimal=5)

    def test_scale_by_vector(self):
        result = rectangle.scale_by_vector([[-1.,-2.],[2.,4.]], [2.,3.])
        np.testing.assert_almost_equal(result, [[-2.,-6.],[4.,12.]], decimal=5)

    def test_scale_by_vector3(self):
        result = rectangle.scale_by_vector([[-1.,-2.],[2.,4.]], [2.,3.,4.])
        np.testing.assert_almost_equal(result, [[-2.,-6.],[4.,12.]], decimal=5)

    def test_right(self):
        result = rectangle.right([[1.,2.],[3.,4.]])
        np.testing.assert_almost_equal(result, 4., decimal=5)

    def test_right_negative(self):
        result = rectangle.right([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, 1., decimal=5)

    def test_left(self):
        result = rectangle.left([[1.,2.],[3.,4.]])
        np.testing.assert_almost_equal(result, 1., decimal=5)

    def test_left_negative(self):
        result = rectangle.left([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, -2., decimal=5)

    def test_top(self):
        result = rectangle.top([[1.,2.],[3.,4.]])
        np.testing.assert_almost_equal(result, 6., decimal=5)

    def test_top_negative(self):
        result = rectangle.top([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, 2., decimal=5)

    def test_bottom(self):
        result = rectangle.bottom([[1.,2.],[3.,4.]])
        np.testing.assert_almost_equal(result, 2., decimal=5)

    def test_bottom_negative(self):
        result = rectangle.bottom([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, -2., decimal=5)

    def test_x(self):
        result = rectangle.x([[1.,2.],[3.,4.]])
        np.testing.assert_almost_equal(result, 1., decimal=5)

    def test_x_negative(self):
        result = rectangle.x([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, 1., decimal=5)

    def test_y(self):
        result = rectangle.y([[1.,2.],[3.,4.]])
        np.testing.assert_almost_equal(result, 2., decimal=5)

    def test_y_negative(self):
        result = rectangle.y([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, 2., decimal=5)

    def test_width(self):
        result = rectangle.width([[1.,2.],[3.,4.]])
        np.testing.assert_almost_equal(result, 3., decimal=5)

    def test_width_negative(self):
        result = rectangle.width([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, -3., decimal=5)

    def test_height(self):
        result = rectangle.height([[1.,2.],[3.,4.]])
        np.testing.assert_almost_equal(result, 4., decimal=5)

    def test_height_negative(self):
        result = rectangle.height([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, -4., decimal=5)

    def test_abs_height(self):
        result = rectangle.abs_height([[1.,2.],[3.,4.]])
        np.testing.assert_almost_equal(result, 4., decimal=5)

    def test_abs_height_negative(self):
        result = rectangle.abs_height([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, 4., decimal=5)

    def test_abs_width(self):
        result = rectangle.abs_width([[1.,2.],[3.,4.]])
        np.testing.assert_almost_equal(result, 3., decimal=5)

    def test_abs_width_negative(self):
        result = rectangle.abs_width([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, 3., decimal=5)

    def test_position(self):
        result = rectangle.position([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, [1.,2.], decimal=5)

    def test_size(self):
        result = rectangle.size([[1.,2.],[-3.,-4.]])
        np.testing.assert_almost_equal(result, [-3.,-4.], decimal=5)

    
if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr.objects.matrix33 import Matrix33
from pyrr.objects.matrix44 import Matrix44
from pyrr.objects.quaternion import Quaternion
from pyrr.objects.vector3 import Vector3
from pyrr.objects.vector4 import Vector4
from pyrr import vector3


class test_object_vector3(unittest.TestCase):
    _shape = (3,)
    _size = np.multiply.reduce(_shape)

    def test_imports(self):
        import pyrr
        pyrr.Vector3()
        pyrr.vector3.Vector3()
        pyrr.objects.vector3.Vector3()

    def test_imports_1(self):
        from pyrr import Vector3
        Vector3

    def test_imports_2(self):
        from pyrr.objects import Vector3
        Vector3

    def test_imports_3(self):
        from pyrr.objects.vector3 import Vector3
        Vector3

    def test_create(self):
        v = Vector3()
        self.assertTrue(np.array_equal(v, [0.,0.,0.]))
        self.assertEqual(v.shape, self._shape)

        v = Vector3([1.,2.,3.])
        self.assertTrue(np.array_equal(v, [1.,2.,3.]))
        self.assertEqual(v.shape, self._shape)

        v = Vector3(Vector3())
        self.assertTrue(np.array_equal(v, [0.,0.,0.]))
        self.assertEqual(v.shape, self._shape)

        v4 = [1., 2., 3., 4.]
        result = vector3.create_from_vector4(v4)
        v, w = result
        np.testing.assert_almost_equal(v, [1.,2.,3.], decimal=5)
        np.testing.assert_almost_equal(w, 4., decimal=5)

        v4 = Vector4([1., 2., 3., 4.])
        result = vector3.create_from_vector4(v4)
        v, w = result
        np.testing.assert_almost_equal(v, [1.,2.,3.], decimal=5)
        np.testing.assert_almost_equal(w, 4., decimal=5)


        m = Matrix44.from_translation([1.,2.,3.])
        v = Vector3.from_matrix44_translation(m)
        self.assertTrue(np.array_equal(v, [1.,2.,3.]))

        m = Matrix44.from_translation([1.,2.,3.])
        v = Vector3(m)
        self.assertTrue(np.array_equal(v, [1.,2.,3.]))

    def test_inverse(self):
        v = Vector3([1.,2.,3.])
        self.assertTrue(np.array_equal(v.inverse, [-1.,-2.,-3.]))

    def test_normalize(self):
        v = Vector3([1.,1.,1.])
        np.testing.assert_almost_equal(np.array(v.normalized), [0.57735, 0.57735, 0.57735], decimal=5)

        v.normalize()
        np.testing.assert_almost_equal(np.array(v), [0.57735, 0.57735, 0.57735], decimal=5)

    def test_operators_matrix33(self):
        v = Vector3()
        m = Matrix33.from_x_rotation(0.5)

        # add
        self.assertRaises(ValueError, lambda: v + m)

        # subtract
        self.assertRaises(ValueError, lambda: v - m)

        # multiply
        self.assertRaises(ValueError, lambda: v - m)

        # divide
        self.assertRaises(ValueError, lambda: v / m)

    def test_operators_matrix44(self):
        v = Vector3()
        m = Matrix44.from_x_rotation(0.5)

        # add
        self.assertRaises(ValueError, lambda: v + m)

        # subtract
        self.assertRaises(ValueError, lambda: v - m)

        # multiply
        self.assertRaises(ValueError, lambda: v * m)

        # divide
        self.assertRaises(ValueError, lambda: v / m)

    def test_operators_quaternion(self):
        v = Vector3()
        q = Quaternion.from_x_rotation(0.5)

        # add
        self.assertRaises(ValueError, lambda: v + q)

        # subtract
        self.assertRaises(ValueError, lambda: v - q)

        # multiply
        self.assertRaises(ValueError, lambda: v * q)

        # divide
        self.assertRaises(ValueError, lambda: v / q)

    def test_operators_vector3(self):
        v1 = Vector3()
        v2 = Vector3([1.,2.,3.])

        # add
        self.assertTrue(np.array_equal(v1 + v2, [1.,2.,3.]))

        # subtract
        self.assertTrue(np.array_equal(v1 - v2, [-1.,-2.,-3.]))

        # multiply
        self.assertTrue(np.array_equal(v1 * v2, [0.,0.,0.]))

        # divide
        self.assertTrue(np.array_equal(v1 / v2, [0.,0.,0.]))

        # or
        self.assertTrue(np.array_equal(v1 | v2, vector3.dot(v1, v2)))

        # xor
        self.assertTrue(np.array_equal(v1 ^ v2, vector3.cross(v1, v2)))

        # ==
        self.assertTrue(Vector3() == Vector3())
        self.assertFalse(Vector3() == Vector3([1.,1.,1.]))

        # !=
        self.assertTrue(Vector3() != Vector3([1.,1.,1.]))
        self.assertFalse(Vector3() != Vector3())

    def test_operators_vector4(self):
        v1 = Vector3()
        v2 = Vector4([1.,2.,3.,4.])

        # add
        self.assertRaises(ValueError, lambda: v1 + v2)

        # subtract
        self.assertRaises(ValueError, lambda: v1 - v2)

        # multiply
        self.assertRaises(ValueError, lambda: v1 * v2)

        # divide
        self.assertRaises(ValueError, lambda: v1 / v2)

        # or
        self.assertRaises(ValueError, lambda: v1 | v2)

        # xor
        self.assertRaises(ValueError, lambda: v1 ^ v2)

        # ==
        self.assertRaises(ValueError, lambda: Vector3() == Vector4())

        # !=
        self.assertRaises(ValueError, lambda: Vector3() != Vector4([1.,1.,1.,1.]))

    def test_operators_number(self):
        v1 = Vector3([1.,2.,3.])
        fv = np.empty((1,), dtype=[('i', np.int16, 1),('f', np.float32, 1)])
        fv[0] = (2, 2.0)

        # add
        self.assertTrue(np.array_equal(v1 + 1., [2., 3., 4.]))
        self.assertTrue(np.array_equal(v1 + 1, [2., 3., 4.]))
        self.assertTrue(np.array_equal(v1 + np.float(1.), [2., 3., 4.]))
        self.assertTrue(np.array_equal(v1 + fv[0]['f'], [3., 4., 5.]))
        self.assertTrue(np.array_equal(v1 + fv[0]['i'], [3., 4., 5.]))

        # subtract
        self.assertTrue(np.array_equal(v1 - 1., [0., 1., 2.]))
        self.assertTrue(np.array_equal(v1 - 1, [0., 1., 2.]))
        self.assertTrue(np.array_equal(v1 - np.float(1.), [0., 1., 2.]))
        self.assertTrue(np.array_equal(v1 - fv[0]['f'], [-1., 0., 1.]))
        self.assertTrue(np.array_equal(v1 - fv[0]['i'], [-1., 0., 1.]))

        # multiply
        self.assertTrue(np.array_equal(v1 * 2., [2., 4., 6.]))
        self.assertTrue(np.array_equal(v1 * 2, [2., 4., 6.]))
        self.assertTrue(np.array_equal(v1 * np.float(2.), [2., 4., 6.]))
        self.assertTrue(np.array_equal(v1 * fv[0]['f'], [2., 4., 6.]))
        self.assertTrue(np.array_equal(v1 * fv[0]['i'], [2., 4., 6.]))

        # divide
        self.assertTrue(np.array_equal(v1 / 2., [.5, 1., 1.5]))
        self.assertTrue(np.array_equal(v1 / 2, [.5, 1., 1.5]))
        self.assertTrue(np.array_equal(v1 / np.float(2.), [.5, 1., 1.5]))
        self.assertTrue(np.array_equal(v1 / fv[0]['f'], [.5, 1., 1.5]))
        self.assertTrue(np.array_equal(v1 / fv[0]['i'], [.5, 1., 1.5]))

        # or
        self.assertRaises(ValueError, lambda: v1 | .5)
        self.assertRaises(ValueError, lambda: v1 | 5)
        self.assertRaises(ValueError, lambda: v1 | np.float(2.))
        self.assertRaises(ValueError, lambda: v1 | fv[0]['f'])
        self.assertRaises(ValueError, lambda: v1 | fv[0]['i'])

        # xor
        self.assertRaises(ValueError, lambda: v1 ^ .5)
        self.assertRaises(ValueError, lambda: v1 ^ 5)
        self.assertRaises(ValueError, lambda: v1 ^ np.float(2.))
        self.assertRaises(ValueError, lambda: v1 ^ fv[0]['f'])
        self.assertRaises(ValueError, lambda: v1 ^ fv[0]['i'])

        # ==
        self.assertRaises(ValueError, lambda: v1 == .5)
        self.assertRaises(ValueError, lambda: v1 == 5)
        self.assertRaises(ValueError, lambda: v1 == np.float(2.))
        self.assertRaises(ValueError, lambda: v1 == fv[0]['f'])
        self.assertRaises(ValueError, lambda: v1 == fv[0]['i'])

        # !=
        self.assertRaises(ValueError, lambda: v1 != .5)
        self.assertRaises(ValueError, lambda: v1 != 5)
        self.assertRaises(ValueError, lambda: v1 != np.float(2.))
        self.assertRaises(ValueError, lambda: v1 != fv[0]['f'])
        self.assertRaises(ValueError, lambda: v1 != fv[0]['i'])

    def test_bitwise(self):
        v1 = Vector3([1.,0.,0.])
        v2 = Vector3([0.,1.,0.])

        # xor (cross)
        self.assertTrue(np.array_equal(v1 ^ v2, vector3.cross(v1, v2)))

        # or (dot)
        self.assertTrue(np.array_equal(v1 | v2, vector3.dot(v1, v2)))

    def test_accessors(self):
        v = Vector3(np.arange(self._size))
        self.assertTrue(np.array_equal(v.xy,[0,1]))
        self.assertTrue(np.array_equal(v.xyz,[0,1,2]))
        self.assertTrue(np.array_equal(v.xz,[0,2]))
        self.assertTrue(np.array_equal(v.xyz,[0,1,2]))

        self.assertEqual(v.x, 0)
        self.assertEqual(v.y, 1)
        self.assertEqual(v.z, 2)

        v.x = 1
        self.assertEqual(v.x, 1)
        self.assertEqual(v[0], 1)
        v.x += 1
        self.assertEqual(v.x, 2)
        self.assertEqual(v[0], 2)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""3x3 Matrix which supports rotation, translation, scale and skew.

Matrices are laid out in row-major format and can be loaded directly
into OpenGL.
To convert to column-major format, transpose the array using the
numpy.array.T method.
"""
from __future__ import absolute_import, division, print_function
import numpy as np
from . import vector, quaternion, euler
from .utils import all_parameters_as_numpy_arrays, parameters_as_numpy_arrays


def create_identity(dtype=None):
    """Creates a new matrix33 and sets it to
    an identity matrix.

    :rtype: numpy.array
    :return: A matrix representing an identity matrix with shape (3,3).
    """
    return np.identity(3, dtype=dtype)

def create_from_matrix44(mat, dtype=None):
    """Creates a Matrix33 from a Matrix44.

    :rtype: numpy.array
    :return: A matrix with shape (3,3) with the input matrix rotation.
    """
    mat = np.asarray(mat)
    return np.array(mat[0:3,0:3], dtype=dtype)

@parameters_as_numpy_arrays('eulers')
def create_from_eulers(eulers, dtype=None):
    """Creates a matrix from the specified Euler rotations.

    :param numpy.array eulers: A set of euler rotations in the format
        specified by the euler modules.
    :rtype: numpy.array
    :return: A matrix with shape (3,3) with the euler's rotation.
    """
    dtype = dtype or eulers.dtype

    pitch, roll, yaw = euler.pitch(eulers), euler.roll(eulers), euler.yaw(eulers)

    sP = np.sin(pitch)
    cP = np.cos(pitch)
    sR = np.sin(roll)
    cR = np.cos(roll)
    sY = np.sin(yaw)
    cY = np.cos(yaw)

    return np.array(
        [
            # m1
            [
                cY * cP,
                -cY * sP * cR + sY * sR,
                cY * sP * sR + sY * cR,
            ],
            # m2
            [
                sP,
                cP * cR,
                -cP * sR,
            ],
            # m3
            [
                -sY * cP,
                sY * sP * cR + cY * sR,
                -sY * sP * sR + cY * cR,
            ]
        ],
        dtype=dtype
    )


@parameters_as_numpy_arrays('axis')
def create_from_axis_rotation(axis, theta, dtype=None):
    """Creates a matrix from the specified theta rotation around an axis.

    :param numpy.array axis: A (3,) vector specifying the axis of rotation.
    :param float theta: A rotation speicified in radians.
    :rtype: numpy.array
    :return: A matrix with shape (3,3).
    """
    dtype = dtype or axis.dtype

    axis = vector.normalize(axis)
    x,y,z = axis

    s = np.sin(theta);
    c = np.cos(theta);
    t = 1 - c;

    # Construct the elements of the rotation matrix
    return np.array(
        [
            [ x * x * t + c,     y * x * t + z * s, z * x * t - y * s],
            [ x * y * t - z * s, y * y * t + c,     z * y * t + x * s],
            [ x * z * t + y * s, y * z * t - x * s, z * z * t + c]
        ],
        dtype= dtype
    )


@parameters_as_numpy_arrays('quat')
def create_from_quaternion(quat, dtype=None):
    """Creates a matrix with the same rotation as a quaternion.

    :param quat: The quaternion to create the matrix from.
    :rtype: numpy.array
    :return: A matrix with shape (3,3) with the quaternion's rotation.
    """
    dtype = dtype or quat.dtype

    # the quaternion must be normalized
    if not np.isclose(np.linalg.norm(quat), 1.):
        quat = quaternion.normalize(quat)

    # http://www.euclideanspace.com/maths/geometry/rotations/conversions/quaternionToMatrix/index.htm
    qx, qy, qz, qw = quat[0], quat[1], quat[2], quat[3]

    sqw = qw**2
    sqx = qx**2
    sqy = qy**2
    sqz = qz**2
    qxy = qx * qy
    qzw = qz * qw
    qxz = qx * qz
    qyw = qy * qw
    qyz = qy * qz
    qxw = qx * qw

    invs = 1 / (sqx + sqy + sqz + sqw)
    m00 = ( sqx - sqy - sqz + sqw) * invs
    m11 = (-sqx + sqy - sqz + sqw) * invs
    m22 = (-sqx - sqy + sqz + sqw) * invs
    m10 = 2.0 * (qxy + qzw) * invs
    m01 = 2.0 * (qxy - qzw) * invs
    m20 = 2.0 * (qxz - qyw) * invs
    m02 = 2.0 * (qxz + qyw) * invs
    m21 = 2.0 * (qyz + qxw) * invs
    m12 = 2.0 * (qyz - qxw) * invs

    return np.array([
        [m00, m01, m02],
        [m10, m11, m12],
        [m20, m21, m22],
    ], dtype=dtype)


@parameters_as_numpy_arrays('quat')
def create_from_inverse_of_quaternion(quat, dtype=None):
    """Creates a matrix with the inverse rotation of a quaternion.

    :param numpy.array quat: The quaternion to make the matrix from (shape 4).
    :rtype: numpy.array
    :return: A matrix with shape (3,3) that respresents the inverse of
        the quaternion.
    """
    dtype = dtype or quat.dtype

    x, y, z, w = quat

    x2 = x**2
    y2 = y**2
    z2 = z**2
    wx = w * x
    wy = w * y
    xy = x * y
    wz = w * z
    xz = x * z
    yz = y * z

    return np.array(
        [
            # m1
            [
                # m11 = 1.0 - 2.0 * (q.y * q.y + q.z * q.z)
                1.0 - 2.0 * (y2 + z2),
                # m21 = 2.0 * (q.x * q.y + q.w * q.z)
                2.0 * (xy + wz),
                # m31 = 2.0 * (q.x * q.z - q.w * q.y)
                2.0 * (xz - wy),
            ],
            # m2
            [
                # m12 = 2.0 * (q.x * q.y - q.w * q.z)
                2.0 * (xy - wz),
                # m22 = 1.0 - 2.0 * (q.x * q.x + q.z * q.z)
                1.0 - 2.0 * (x2 + z2),
                # m32 = 2.0 * (q.y * q.z + q.w * q.x)
                2.0 * (yz + wx),
            ],
            # m3
            [
                # m13 = 2.0 * ( q.x * q.z + q.w * q.y)
                2.0 * (xz + wy),
                # m23 = 2.0 * (q.y * q.z - q.w * q.x)
                2.0 * (yz - wx),
                # m33 = 1.0 - 2.0 * (q.x * q.x + q.y * q.y)
                1.0 - 2.0 * (x2 + y2),
            ]
        ],
        dtype=dtype
    )

def create_from_scale(scale, dtype=None):
    """Creates an identity matrix with the scale set.

    :param numpy.array scale: The scale to apply as a vector (shape 3).
    :rtype: numpy.array
    :return: A matrix with shape (3,3) with the scale
        set to the specified vector.
    """
    # apply the scale to the values diagonally
    # down the matrix
    m = np.diagflat(scale)
    if dtype:
        m = m.astype(dtype)
    return m

def create_from_x_rotation(theta, dtype=None):
    """Creates a matrix with the specified rotation about the X axis.

    :param float theta: The rotation, in radians, about the X-axis.
    :rtype: numpy.array
    :return: A matrix with the shape (3,3) with the specified rotation about
        the X-axis.

    .. seealso:: http://en.wikipedia.org/wiki/Rotation_matrix#In_three_dimensions
    """
    cosT = np.cos(theta)
    sinT = np.sin(theta)

    return np.array(
        [
            [ 1.0, 0.0, 0.0 ],
            [ 0.0, cosT,-sinT ],
            [ 0.0, sinT, cosT ]
        ],
        dtype=dtype
    )

def create_from_y_rotation(theta, dtype=None):
    """Creates a matrix with the specified rotation about the Y axis.

    :param float theta: The rotation, in radians, about the Y-axis.
    :rtype: numpy.array
    :return: A matrix with the shape (3,3) with the specified rotation about
        the Y-axis.

    .. seealso:: http://en.wikipedia.org/wiki/Rotation_matrix#In_three_dimensions
    """
    cosT = np.cos(theta)
    sinT = np.sin(theta)

    return np.array(
        [
            [ cosT, 0.0,sinT ],
            [ 0.0, 1.0, 0.0 ],
            [-sinT, 0.0, cosT ]
        ],
        dtype=dtype
    )

def create_from_z_rotation(theta, dtype=None):
    """Creates a matrix with the specified rotation about the Z axis.

    :param float theta: The rotation, in radians, about the Z-axis.
    :rtype: numpy.array
    :return: A matrix with the shape (3,3) with the specified rotation about
        the Z-axis.

    .. seealso:: http://en.wikipedia.org/wiki/Rotation_matrix#In_three_dimensions
    """
    cosT = np.cos(theta)
    sinT = np.sin(theta)

    return np.array(
        [
            [ cosT,-sinT, 0.0 ],
            [ sinT, cosT, 0.0 ],
            [ 0.0, 0.0, 1.0 ]
        ],
        dtype=dtype
    )

@parameters_as_numpy_arrays('vec')
def apply_to_vector(mat, vec):
    """Apply a matrix to a vector.

    The matrix's rotation are applied to the vector.
    Supports multiple matrices and vectors.

    :param numpy.array mat: The rotation / translation matrix.
        Can be a list of matrices.
    :param numpy.array vec: The vector to modify.
        Can be a list of vectors.
    :rtype: numpy.array
    :return: The vectors rotated by the specified matrix.
    """
    if vec.size == 3:
        return np.dot(vec, mat)
    else:
        raise ValueError("Vector size unsupported")

def multiply(m1, m2):
    """Multiply two matricies, m1 . m2.

    This is essentially a wrapper around
    numpy.dot( m1, m2 )

    :param numpy.array m1: The first matrix.
        Can be a list of matrices.
    :param numpy.array m2: The second matrix.
        Can be a list of matrices.
    :rtype: numpy.array
    :return: A matrix that results from multiplying m1 by m2.
    """
    return np.dot(m1, m2)

def inverse(mat):
    """Returns the inverse of the matrix.

    This is essentially a wrapper around numpy.linalg.inv.

    :param numpy.array m: A matrix.
    :rtype: numpy.array
    :return: The inverse of the specified matrix.

    .. seealso:: http://docs.scipy.org/doc/numpy/reference/generated/numpy.linalg.inv.html
    """
    return np.linalg.inv(mat)

def create_direction_scale(direction, scale):
    """Creates a matrix which can apply a directional scaling to a set of vectors.

    An example usage for this is to flatten a mesh against a
    single plane.

    :param numpy.array direction: a numpy.array of shape (3,) of the direction to scale.
    :param float scale: a float value for the scaling along the specified direction.
        A scale of 0.0 will flatten the vertices into a single plane with the direction being the
        plane's normal.
    :rtype: numpy.array
    :return: The scaling matrix.
    """
    """
    scaling is defined as:

    [p'][1 + (k - 1)n.x^2, (k - 1)n.x n.y^2, (k - 1)n.x n.z   ]
    S(n,k) = [q'][(k - 1)n.x n.y,   1 + (k - 1)n.y,   (k - 1)n.y n.z   ]
    [r'][(k - 1)n.x n.z,   (k - 1)n.y n.z,   1 + (k - 1)n.z^2 ]

    where:
    v' is the resulting vector after scaling
    v is the vector to scale
    n is the direction of the scaling
    n.x is the x component of n
    n.y is the y component of n
    n.z is the z component of n
    k is the scaling factor
    """
    if not np.isclose(np.linalg.norm(direction), 1.):
        direction = vector.normalize(direction)

    x,y,z = direction

    x2 = x**2.
    y2 = y**2.
    z2 = z**2

    scaleMinus1 = scale - 1.
    return np.array(
        [
            # m1
            [
                # m11 = 1 + (k - 1)n.x^2
                1. + scaleMinus1 * x2,
                # m12 = (k - 1)n.x n.y^2
                scaleMinus1 * x * y2,
                # m13 = (k - 1)n.x n.z
                scaleMinus1 * x * z
            ],
            # m2
            [
                # m21 = (k - 1)n.x n.y
                scaleMinus1 * x * y,
                # m22 = 1 + (k - 1)n.y
                1. + scaleMinus1 * y,
                # m23 = (k - 1)n.y n.z
                scaleMinus1 * y * z
            ],
            # m3
            [
                # m31 = (k - 1)n.x n.z
                scaleMinus1 * x * z,
                # m32 = (k - 1)n.y n.z
                scaleMinus1 * y * z,
                # m33 = 1 + (k - 1)n.z^2
                1. + scaleMinus1 * z2
            ]
        ]
    )



from __future__ import absolute_import
try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr.objects.matrix33 import Matrix33
from pyrr.objects.matrix44 import Matrix44
from pyrr.objects.quaternion import Quaternion
from pyrr.objects.vector3 import Vector3
from pyrr.objects.vector4 import Vector4
from pyrr import vector4


class test_object_vector4(unittest.TestCase):
    _shape = (4,)
    _size = np.multiply.reduce(_shape)

    def test_imports(self):
        import pyrr
        pyrr.Vector4()
        pyrr.vector4.Vector4()
        pyrr.objects.vector4.Vector4()

        from pyrr import Vector4
        from pyrr.objects import Vector4
        from pyrr.objects.vector4 import Vector4

    def test_create(self):
        v = Vector4()
        self.assertTrue(np.array_equal(v, [0.,0.,0.,0.]))
        self.assertEqual(v.shape, self._shape)

        v = Vector4([1.,2.,3.,4.])
        self.assertTrue(np.array_equal(v, [1.,2.,3.,4.]))
        self.assertEqual(v.shape, self._shape)

        v = Vector4.from_vector3([1.,2.,3.], w=0.0)
        self.assertTrue(np.array_equal(v, [1.,2.,3.,0.]))
        self.assertEqual(v.shape, self._shape)

        v = Vector4(Vector4())
        self.assertTrue(np.array_equal(v, [0.,0.,0.,0.]))
        self.assertEqual(v.shape, self._shape)

        m = Matrix44.from_translation([1.,2.,3.])
        v = Vector4.from_matrix44_translation(m)
        self.assertTrue(np.array_equal(v, [1.,2.,3.,1.]))

        m = Matrix44.from_translation([1.,2.,3.])
        v = Vector4(m)
        self.assertTrue(np.array_equal(v, [1.,2.,3.,1.]))

    def test_inverse(self):
        v = Vector4([1.,2.,3.,4.])
        self.assertTrue(np.array_equal(v.inverse, [-1.,-2.,-3.,-4.]))

    def test_normalize(self):
        v = Vector4([1.,1.,1.,1.])
        np.testing.assert_almost_equal(np.array(v.normalized), [0.5, 0.5, 0.5, 0.5], decimal=5)

        v.normalize()
        np.testing.assert_almost_equal(np.array(v), [0.5, 0.5, 0.5, 0.5], decimal=5)

    def test_operators_matrix33(self):
        v = Vector4()
        m = Matrix33.from_x_rotation(0.5)

        # add
        self.assertRaises(ValueError, lambda: v + m)

        # subtract
        self.assertRaises(ValueError, lambda: v - m)

        # multiply
        self.assertRaises(ValueError, lambda: v - m)

        # divide
        self.assertRaises(ValueError, lambda: v / m)

    def test_operators_matrix44(self):
        v = Vector4()
        m = Matrix44.from_x_rotation(0.5)

        # add
        self.assertRaises(ValueError, lambda: v + m)

        # subtract
        self.assertRaises(ValueError, lambda: v - m)

        # multiply
        self.assertRaises(ValueError, lambda: v * m)

        # divide
        self.assertRaises(ValueError, lambda: v / m)

    def test_operators_quaternion(self):
        v = Vector4()
        q = Quaternion.from_x_rotation(0.5)

        # add
        self.assertRaises(ValueError, lambda: v + q)

        # subtract
        self.assertRaises(ValueError, lambda: v - q)

        # multiply
        self.assertRaises(ValueError, lambda: v * q)

        # divide
        self.assertRaises(ValueError, lambda: v / q)

    def test_operators_vector3(self):
        v1 = Vector4()
        v2 = Vector3([1.,2.,3.])

        # add
        self.assertRaises(ValueError, lambda: v1 + v2)

        # subtract
        self.assertRaises(ValueError, lambda: v1 - v2)

        # multiply
        self.assertRaises(ValueError, lambda: v1 * v2)

        # divide
        #self.assertRaises(ValueError, lambda: v1 / v2)

        # or
        self.assertRaises(ValueError, lambda: v1 | v2)

        # xor
        #self.assertRaises(ValueError, lambda: v1 ^ v2)

        # ==
        self.assertRaises(ValueError, lambda: Vector4() == Vector3())

        # !=
        self.assertRaises(ValueError, lambda: Vector4() != Vector3([1.,1.,1.]))

    def test_operators_vector4(self):
        v1 = Vector4()
        v2 = Vector4([1.,2.,3.,4.])

        # add
        self.assertTrue(np.array_equal(v1 + v2, [1.,2.,3.,4.]))

        # subtract
        self.assertTrue(np.array_equal(v1 - v2, [-1.,-2.,-3.,-4]))

        # multiply
        self.assertTrue(np.array_equal(v1 * v2, [0.,0.,0.,0.]))

        # divide
        self.assertTrue(np.array_equal(v1 / v2, [0.,0.,0.,0.]))

        # or
        self.assertTrue(np.array_equal(v1 | v2, vector4.dot([0.,0.,0.,0.], [1.,2.,3.,4.])))

        # xor
        #self.assertTrue(np.array_equal(v1 ^ v2, vector4.cross([0.,0.,0.,0.], [1.,2.,3.,4.])))

        # ==
        self.assertTrue(Vector4() == Vector4())
        self.assertFalse(Vector4() == Vector4([1.,1.,1.,1.]))

        # !=
        self.assertTrue(Vector4() != Vector4([1.,1.,1.,1.]))
        self.assertFalse(Vector4() != Vector4())

    def test_operators_number(self):
        v1 = Vector4([1.,2.,3.,4.])
        fv = np.empty((1,), dtype=[('i', np.int16, 1),('f', np.float32, 1)])
        fv[0] = (2, 2.0)

        # add
        self.assertTrue(np.array_equal(v1 + 1., [2., 3., 4., 5.]))
        self.assertTrue(np.array_equal(v1 + 1, [2., 3., 4., 5.]))
        self.assertTrue(np.array_equal(v1 + np.float(1.), [2., 3., 4., 5.]))
        self.assertTrue(np.array_equal(v1 + fv[0]['f'], [3., 4., 5., 6.]))
        self.assertTrue(np.array_equal(v1 + fv[0]['i'], [3., 4., 5., 6.]))

        # subtract
        self.assertTrue(np.array_equal(v1 - 1., [0., 1., 2., 3.]))
        self.assertTrue(np.array_equal(v1 - 1, [0., 1., 2., 3.]))
        self.assertTrue(np.array_equal(v1 - np.float(1.), [0., 1., 2., 3.]))
        self.assertTrue(np.array_equal(v1 - fv[0]['f'], [-1., 0., 1., 2.]))
        self.assertTrue(np.array_equal(v1 - fv[0]['i'], [-1., 0., 1., 2.]))

        # multiply
        self.assertTrue(np.array_equal(v1 * 2., [2., 4., 6., 8.]))
        self.assertTrue(np.array_equal(v1 * 2, [2., 4., 6., 8.]))
        self.assertTrue(np.array_equal(v1 * np.float(2.), [2., 4., 6., 8.]))
        self.assertTrue(np.array_equal(v1 * fv[0]['f'], [2., 4., 6., 8.]))
        self.assertTrue(np.array_equal(v1 * fv[0]['i'], [2., 4., 6., 8.]))

        # divide
        self.assertTrue(np.array_equal(v1 / 2., [.5, 1., 1.5, 2.]))
        self.assertTrue(np.array_equal(v1 / 2, [.5, 1., 1.5, 2.]))
        self.assertTrue(np.array_equal(v1 / np.float(2.), [.5, 1., 1.5, 2.]))
        self.assertTrue(np.array_equal(v1 / fv[0]['f'], [.5, 1., 1.5, 2.]))
        self.assertTrue(np.array_equal(v1 / fv[0]['i'], [.5, 1., 1.5, 2.]))

        # or
        self.assertRaises(ValueError, lambda: v1 | .5)
        self.assertRaises(ValueError, lambda: v1 | 5)
        self.assertRaises(ValueError, lambda: v1 | np.float(2.))
        self.assertRaises(ValueError, lambda: v1 | fv[0]['f'])
        self.assertRaises(ValueError, lambda: v1 | fv[0]['i'])

        # xor
        self.assertRaises(ValueError, lambda: v1 ^ .5)
        self.assertRaises(ValueError, lambda: v1 ^ 5)
        self.assertRaises(ValueError, lambda: v1 ^ np.float(2.))
        self.assertRaises(ValueError, lambda: v1 ^ fv[0]['f'])
        self.assertRaises(ValueError, lambda: v1 ^ fv[0]['i'])

        # ==
        self.assertRaises(ValueError, lambda: v1 == .5)
        self.assertRaises(ValueError, lambda: v1 == 5)
        self.assertRaises(ValueError, lambda: v1 == np.float(2.))
        self.assertRaises(ValueError, lambda: v1 == fv[0]['f'])
        self.assertRaises(ValueError, lambda: v1 == fv[0]['i'])

        # !=
        self.assertRaises(ValueError, lambda: v1 != .5)
        self.assertRaises(ValueError, lambda: v1 != 5)
        self.assertRaises(ValueError, lambda: v1 != np.float(2.))
        self.assertRaises(ValueError, lambda: v1 != fv[0]['f'])
        self.assertRaises(ValueError, lambda: v1 != fv[0]['i'])

    def test_bitwise(self):
        v1 = Vector4([1.,0.,0.,1.])
        v2 = Vector4([0.,1.,0.,1.])

        # or (dot)
        self.assertTrue(np.array_equal(v1 | v2, vector4.dot(v1, v2)))

    def test_accessors(self):
        v = Vector4(np.arange(self._size))
        self.assertTrue(np.array_equal(v.xy,[0,1]))
        self.assertTrue(np.array_equal(v.xyz,[0,1,2]))
        self.assertTrue(np.array_equal(v.xz,[0,2]))
        self.assertTrue(np.array_equal(v.xyz,[0,1,2]))

        self.assertEqual(v.x, 0)
        self.assertEqual(v.y, 1)
        self.assertEqual(v.z, 2)

        v.x = 1
        self.assertEqual(v.x, 1)
        self.assertEqual(v[0], 1)
        v.x += 1
        self.assertEqual(v.x, 2)
        self.assertEqual(v[0], 2)

if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import, division, print_function
try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import Quaternion, Matrix44, Matrix33, Vector3, Vector4, euler


class test_matrix_quaternion(unittest.TestCase):
    def test_m44_q_equivalence(self):
        """Test for equivalance of matrix and quaternion rotations.

        Create a matrix and quaternion, rotate each by the same values
        then convert matrix<->quaternion and check the results are the same.
        """
        m = Matrix44.from_x_rotation(np.pi / 2.)
        mq = Quaternion.from_matrix(m)

        q = Quaternion.from_x_rotation(np.pi / 2.)
        qm = Matrix44.from_quaternion(q)

        self.assertTrue(np.allclose(np.dot([1., 0., 0., 1.], m), [1., 0., 0., 1.]))
        self.assertTrue(np.allclose(np.dot([1., 0., 0., 1.], qm), [1., 0., 0., 1.]))

        self.assertTrue(np.allclose(q * Vector4([1., 0., 0., 1.]), [1., 0., 0., 1.]))
        self.assertTrue(np.allclose(mq * Vector4([1., 0., 0., 1.]), [1., 0., 0., 1.]))

        np.testing.assert_almost_equal(np.array(q), np.array(mq), decimal=5)
        np.testing.assert_almost_equal(np.array(m), np.array(qm), decimal=5)

    def test_euler_equivalence(self):
        eulers = euler.create_from_x_rotation(np.pi / 2.)
        m = Matrix33.from_x_rotation(np.pi / 2.)
        q = Quaternion.from_x_rotation(np.pi / 2.)
        qm = Matrix33.from_quaternion(q)
        em = Matrix33.from_eulers(eulers)
        self.assertTrue(np.allclose(qm, m))
        self.assertTrue(np.allclose(qm, em))
        self.assertTrue(np.allclose(m, em))

    def test_quaternion_matrix_conversion(self):
        # https://au.mathworks.com/help/robotics/ref/quat2rotm.html?requestedDomain=www.mathworks.com
        q = Quaternion([0.7071, 0., 0., 0.7071])
        m33 = q.matrix33
        expected = np.array([
            [1., 0., 0.],
            [0.,-0.,-1.],
            [0., 1.,-0.],
        ])
        self.assertTrue(np.allclose(m33, expected))

        # issue #42
        q = Quaternion([0.80087974, 0.03166748, 0.59114721,-0.09018753])
        m33 = q.matrix33
        q2 = Quaternion.from_matrix(m33)
        print(q, q2)
        self.assertTrue(np.allclose(q, q2))

        q3 = Quaternion.from_matrix(m33.T)
        self.assertFalse(np.allclose(q2, q3))


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
try:
    import unittest2 as unittest
except:
    import unittest


class test_oo_examples(unittest.TestCase):
    def test_oo_examples(self):
        from pyrr import Quaternion, Matrix44, Vector3
        import numpy as np

        point = Vector3([1.,2.,3.])
        orientation = Quaternion()
        translation = Vector3()
        scale = Vector3([1.,1.,1.])

        # translate along X by 1
        translation += [1.0, 0.0, 0.0]

        # rotate about Y by pi/2
        rotation = Quaternion.from_y_rotation(np.pi / 2.0)
        orientation = rotation * orientation

        # create a matrix
        # start our matrix off using the scale
        matrix = Matrix44.from_scale(scale)

        # apply our orientation
        # we can multiply matricies and quaternions directly!
        matrix = matrix * orientation

        # apply our translation
        translation = Matrix44.from_translation(translation)
        matrix = matrix * translation

        # transform our point by the matrix
        # vectors are transformable by matrices and quaternions directly
        point = matrix * point

    def test_conversions(self):
        from pyrr import Quaternion, Matrix33, Matrix44, Vector3, Vector4

        v3 = Vector3([1.,0.,0.])
        v4 = Vector4.from_vector3(v3, w=1.0)
        v3, w = Vector3.from_vector4(v4)

        m44 = Matrix44()
        q = Quaternion(m44)
        m33 = Matrix33(q)

        m33 = Matrix44().matrix33
        m44 = Matrix33().matrix44
        q = Matrix44().quaternion
        q = Matrix33().quaternion

        m33 = Quaternion().matrix33
        m44 = Quaternion().matrix44

    def test_operators(self):
        from pyrr import Quaternion, Matrix44, Matrix33, Vector3, Vector4
        import numpy as np

        # matrix multiplication
        m = Matrix44() * Matrix33()
        m = Matrix44() * Quaternion()
        m = Matrix33() * Quaternion()

        # matrix inverse
        m = ~Matrix44.from_x_rotation(np.pi)

        # quaternion multiplication
        q = Quaternion() * Quaternion()
        q = Quaternion() * Matrix44()
        q = Quaternion() * Matrix33()

        # quaternion inverse (conjugate)
        q = ~Quaternion()

        # quaternion dot product
        d = Quaternion() | Quaternion()

        # vector oprations
        v = Vector3() + Vector3()
        v = Vector4() - Vector4()

        # vector transform
        v = Quaternion() * Vector3()
        v = Matrix44() * Vector3()
        v = Matrix44() * Vector4()
        v = Matrix33() * Vector3()

        # dot and cross products
        dot = Vector3() | Vector3()
        cross = Vector3() ^ Vector3()



# -*- coding: utf-8 -*-
"""Represents a 3x3 Matrix.

The Matrix33 class provides a number of convenient functions and
conversions.
::

    import numpy as np
    from pyrr import Quaternion, Matrix33, Matrix44, Vector3

    m = Matrix33()
    m = Matrix33([[1.,0.,0.],[0.,1.,0.],[0.,0.,1.]])

    # copy constructor
    m = Matrix44(Matrix44())

    # explicit creation
    m = Matrix33.identity()
    m = Matrix33.from_matrix44(Matrix44())

    # inferred conversions
    m = Matrix33(Quaternion())
    m = Matrix33(Matrix44())

    # multiply matricies together
    m = Matrix33() * Matrix33()
    m = Matrix33() * Matrix44()

    # extract a quaternion from a matrix
    q = m.quaternion

    # convert from quaternion back to matrix
    m = q.matrix33
    m = Matrix33(q)

    # rotate a matrix by a quaternion
    m = Matrix33.identity() * Quaternion()

    # rotate a vector 3 by a matrix
    v = Matrix33.from_x_rotation(np.pi) * Vector3([1.,2.,3.])

    # undo a rotation
    m = Matrix33.from_x_rotation(np.pi)
    v = m * Vector3([1.,1.,1.])
    # ~m is the same as m.inverse
    v = ~m * v

    # access specific parts of the matrix
    # first row
    m1 = m.m1
    # first element, first row
    m11 = m.m11
    # third element, third row
    m33 = m.m33
    # first row, same as m1
    r1 = m.r1
    # first column
    c1 = m.c1
"""
from __future__ import absolute_import
from numbers import Number
import numpy as np
from multipledispatch import dispatch
from .base import BaseObject, BaseMatrix, BaseMatrix33, BaseQuaternion, BaseVector, NpProxy
from .. import matrix33

class Matrix33(BaseMatrix33):
    _module = matrix33
    _shape = (3,3,)

    # m<c> style access
    #: The first row of this Matrix as a numpy.ndarray.
    m1 = NpProxy(0)
    #: The second row of this Matrix as a numpy.ndarray.
    m2 = NpProxy(1)
    #: The third row of this Matrix as a numpy.ndarray.
    m3 = NpProxy(2)

    # m<r><c> access
    #: The [0,0] value of this Matrix.
    m11 = NpProxy((0,0))
    #: The [0,1] value of this Matrix.
    m12 = NpProxy((0,1))
    #: The [0,2] value of this Matrix.
    m13 = NpProxy((0,2))
    #: The [1,0] value of this Matrix.
    m21 = NpProxy((1,0))
    #: The [1,1] value of this Matrix.
    m22 = NpProxy((1,1))
    #: The [1,2] value of this Matrix.
    m23 = NpProxy((1,2))
    #: The [2,0] value of this Matrix.
    m31 = NpProxy((2,0))
    #: The [2,1] value of this Matrix.
    m32 = NpProxy((2,1))
    #: The [2,2] value of this Matrix.
    m33 = NpProxy((2,2))

    # rows
    #: The first row of this Matrix as a numpy.ndarray. This is the same as m1.
    r1 = NpProxy(0)
    #: The second row of this Matrix as a numpy.ndarray. This is the same as m2.
    r2 = NpProxy(1)
    #: The third row of this Matrix as a numpy.ndarray. This is the same as m3.
    r3 = NpProxy(2)

    # columns
    #: The first column of this Matrix as a numpy.ndarray.
    c1 = NpProxy((slice(0,3),0))
    #: The second column of this Matrix as a numpy.ndarray.
    c2 = NpProxy((slice(0,3),1))
    #: The third column of this Matrix as a numpy.ndarray.
    c3 = NpProxy((slice(0,3),2))

    ########################
    # Creation
    @classmethod
    def from_matrix44(cls, matrix, dtype=None):
        """Creates a Matrix33 from a Matrix44.

        The Matrix44 translation will be lost.
        """
        return cls(matrix33.create_from_matrix44(matrix, dtype))

    def __new__(cls, value=None, dtype=None):
        if value is not None:
            obj = value
            if not isinstance(value, np.ndarray):
                obj = np.array(value, dtype=dtype)

            # matrix44
            if obj.shape == (4,4) or isinstance(obj, Matrix44):
                obj = matrix33.create_from_matrix44(obj, dtype=dtype)
            # quaternion
            elif obj.shape == (4,) or isinstance(obj, Quaternion):
                obj = matrix33.create_from_quaternion(obj, dtype=dtype)
        else:
            obj = np.zeros(cls._shape, dtype=dtype)
        obj = obj.view(cls)
        return super(Matrix33, cls).__new__(cls, obj)

    ########################
    # Basic Operators

    @dispatch(BaseObject)
    def __add__(self, other):
        self._unsupported_type('add', other)

    @dispatch(BaseObject)
    def __sub__(self, other):
        self._unsupported_type('subtract', other)

    @dispatch(BaseObject)
    def __mul__(self, other):
        self._unsupported_type('multiply', other)

    @dispatch(BaseObject)
    def __truediv__(self, other):
        self._unsupported_type('divide', other)

    @dispatch(BaseObject)
    def __div__(self, other):
        self._unsupported_type('divide', other)

    def __invert__(self):
        return self.inverse

    ########################
    # Matrices
    @dispatch((BaseMatrix, np.ndarray, list))
    def __add__(self, other):
        return Matrix33(super(Matrix33, self).__add__(Matrix33(other)))

    @dispatch((BaseMatrix, np.ndarray, list))
    def __sub__(self, other):
        return Matrix33(super(Matrix33, self).__sub__(Matrix33(other)))

    @dispatch((BaseMatrix, np.ndarray, list))
    def __mul__(self, other):
        return Matrix33(matrix33.multiply(Matrix33(other), self))

    @dispatch((BaseMatrix, np.ndarray, list))
    def __ne__(self, other):
        return bool(np.any(super(Matrix33, self).__ne__(other)))

    @dispatch((BaseMatrix, np.ndarray, list))
    def __eq__(self, other):
        return bool(np.all(super(Matrix33, self).__eq__(other)))

    ########################
    # Quaternions
    @dispatch(BaseQuaternion)
    def __mul__(self, other):
        m = other.matrix33
        return self * m

    ########################
    # Vectors
    @dispatch(BaseVector)
    def __mul__(self, other):
        return type(other)(matrix33.apply_to_vector(self, other))

    ########################
    # Number
    @dispatch((Number, np.number))
    def __add__(self, other):
        return Matrix33(super(Matrix33, self).__add__(other))

    @dispatch((Number, np.number))
    def __sub__(self, other):
        return Matrix33(super(Matrix33, self).__sub__(other))

    @dispatch((Number, np.number))
    def __mul__(self, other):
        return Matrix33(super(Matrix33, self).__mul__(other))

    @dispatch((Number, np.number))
    def __truediv__(self, other):
        return Matrix33(super(Matrix33, self).__truediv__(other))

    @dispatch((Number, np.number))
    def __div__(self, other):
        return Matrix33(super(Matrix33, self).__div__(other))

    ########################
    # Methods and Properties
    @property
    def matrix33(self):
        """Returns the Matrix33.

        This can be handy if you're not sure what type of Matrix class you have
        but require a Matrix33.
        """
        return self

    @property
    def matrix44(self):
        """Returns a Matrix44 representing this matrix.
        """
        return Matrix44(self)

    @property
    def quaternion(self):
        """Returns a Quaternion representing this matrix.
        """
        return Quaternion(self)

from .matrix44 import Matrix44
from .quaternion import Quaternion



try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import matrix33, quaternion, vector3


class test_matrix33(unittest.TestCase):
    # use wolfram alpha to get information on quaternion conversion values
    # be aware that wolfram lists it as w,x,y,z
    def test_import(self):
        import pyrr
        pyrr.matrix33
        from pyrr import matrix33

    def test_create_from_quaternion_unit(self):
        result = matrix33.create_from_quaternion([0.,0.,0.,1.])
        np.testing.assert_almost_equal(result, np.eye(3), decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_quaternion_x(self):
        result = matrix33.create_from_quaternion([1.,0.,0.,0.])
        expected = [
            [1.,0.,0.],
            [0.,-1.,0.],
            [0.,0.,-1.],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_quaternion_y(self):
        result = matrix33.create_from_quaternion([0.,1.,0.,0.])
        expected = [
            [-1.,0.,0.],
            [0.,1.,0.],
            [0.,0.,-1.],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_quaternion_z(self):
        result = matrix33.create_from_quaternion([0.,0.,1.,0.])
        expected = [
            [-1.,0.,0.],
            [0.,-1.,0.],
            [0.,0.,1.],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_quaternion_rotation(self):
        result = matrix33.create_from_quaternion([.57735,.57735,.57735,0.])
        expected = [
            [-0.333333, 0.666667, 0.666667],
            [0.666667, -0.333333, 0.666667],
            [0.666667, 0.666667, -0.333333],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_quaternion_equivalent(self):
        result = matrix33.create_from_quaternion(quaternion.create_from_x_rotation(0.5))
        expected = matrix33.create_from_x_rotation(0.5)
        np.testing.assert_almost_equal(result, expected, decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_inverse_equivalence(self):
        q = [5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17]
        result = matrix33.create_from_quaternion(quaternion.inverse(q))
        expected = matrix33.inverse(matrix33.create_from_quaternion(q))
        np.testing.assert_almost_equal(result, expected, decimal=5)

        q = quaternion.create_from_x_rotation(0.5)
        result = matrix33.create_from_inverse_of_quaternion(q)
        expected = matrix33.inverse(matrix33.create_from_quaternion(q))
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_create_from_inverse_of_quaternion(self):
        q = quaternion.create_from_x_rotation(np.pi / 2.0)
        result = matrix33.create_from_inverse_of_quaternion(q)
        self.assertTrue(np.allclose(result, matrix33.create_from_x_rotation(-np.pi / 2.0)))

    def test_create_identity(self):
        result = matrix33.create_identity()
        np.testing.assert_almost_equal(result, np.eye(3), decimal=5)
        self.assertTrue(result.dtype == np.float)

    def create_from_matrix44(self):
        m44 = np.arange((4,4))
        result = matrix33.create_from_matrix44(m44)
        self.assertTrue(np.allclose(result, m44[:3][:3]))

    @unittest.skip('Not implemented')
    def test_create_from_eulers(self):
        # just call the function
        # TODO: check the result
        matrix33.create_from_eulers([1,2,3])

    def test_create_from_axis_rotation(self):
        # wolfram alpha can be awesome sometimes
        result = matrix33.create_from_axis_rotation([0.57735, 0.57735, 0.57735],np.pi)
        np.testing.assert_almost_equal(result, matrix33.create_from_quaternion([5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17]), decimal=3)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_axis_rotation_non_normalized(self):
        result = matrix33.create_from_axis_rotation([1.,1.,1.], np.pi)
        np.testing.assert_almost_equal(result, matrix33.create_from_quaternion([5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17]), decimal=3)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_x_rotation(self):
        mat = matrix33.create_from_x_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(np.dot([1.,0.,0.], mat), [1.,0.,0.]))
        self.assertTrue(np.allclose(np.dot([0.,1.,0.], mat), [0.,0.,-1.]))
        self.assertTrue(np.allclose(np.dot([0.,0.,1.], mat), [0.,1.,0.]))

    def test_create_from_y_rotation(self):
        mat = matrix33.create_from_y_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(np.dot([1.,0.,0.], mat), [0.,0.,1.]))
        self.assertTrue(np.allclose(np.dot([0.,1.,0.], mat), [0.,1.,0.]))
        self.assertTrue(np.allclose(np.dot([0.,0.,1.], mat), [-1.,0.,0.]))

    def test_create_from_z_rotation(self):
        mat = matrix33.create_from_z_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(np.dot([1.,0.,0.], mat), [0.,-1.,0.]))
        self.assertTrue(np.allclose(np.dot([0.,1.,0.], mat), [1.,0.,0.]))
        self.assertTrue(np.allclose(np.dot([0.,0.,1.], mat), [0.,0.,1.]))

    def test_create_from_scale(self):
        scale = np.array([ 2.0, 3.0, 4.0])
        mat = matrix33.create_from_scale(scale)
        result = mat.diagonal()
        expected = scale
        self.assertTrue(np.array_equal(result, expected))

    def test_create_from_quaternion_identity(self):
        quat = quaternion.create()
        result = matrix33.create_from_quaternion(quat)
        expected = np.eye(3)
        self.assertTrue(np.array_equal(result, expected))

    def test_create_from_quaternion_rotated_x(self):
        quat = quaternion.create_from_x_rotation(np.pi)
        result = matrix33.create_from_quaternion(quat)
        expected = matrix33.create_from_x_rotation(np.pi)
        self.assertTrue(np.allclose(result, expected))

    def test_create_from_quaternion_rotated_y(self):
        quat = quaternion.create_from_y_rotation(np.pi)
        result = matrix33.create_from_quaternion(quat)
        expected = matrix33.create_from_y_rotation(np.pi)
        self.assertTrue(np.allclose(result, expected))

    def test_create_from_quaternion_rotated_z(self):
        quat = quaternion.create_from_z_rotation(np.pi)
        result = matrix33.create_from_quaternion(quat)
        expected = matrix33.create_from_z_rotation(np.pi)
        self.assertTrue(np.allclose(result, expected))

    def test_apply_to_vector_identity(self):
        mat = matrix33.create_identity()
        vec = vector3.unit.x
        result = matrix33.apply_to_vector(mat, vec)
        expected = vec
        self.assertTrue(np.array_equal(result, expected))

    def test_apply_to_vector_rotated_x(self):
        mat = matrix33.create_from_x_rotation(np.pi)
        vec = vector3.unit.y
        result = matrix33.apply_to_vector(mat, vec)
        expected = -vec
        self.assertTrue(np.allclose(result, expected))

    def test_apply_to_vector_rotated_y(self):
        mat = matrix33.create_from_y_rotation(np.pi)
        vec = vector3.unit.x
        result = matrix33.apply_to_vector(mat, vec)
        expected = -vec
        self.assertTrue(np.allclose(result, expected))

    def test_apply_to_vector_rotated_z(self):
        mat = matrix33.create_from_z_rotation(np.pi)
        vec = vector3.unit.x
        result = matrix33.apply_to_vector(mat, vec)
        expected = -vec
        self.assertTrue(np.allclose(result, expected))

    def test_multiply_identity(self):
        m1 = matrix33.create_identity()
        m2 = matrix33.create_identity()
        result = matrix33.multiply(m1, m2)
        self.assertTrue(np.allclose(result, np.dot(m1,m2)))

    def test_multiply_rotation(self):
        m1 = matrix33.create_from_x_rotation(np.pi)
        m2 = matrix33.create_from_y_rotation(np.pi / 2.0)
        result = matrix33.multiply(m1, m2)
        self.assertTrue(np.allclose(result, np.dot(m1,m2)))

    def test_inverse(self):
        m = matrix33.create_from_y_rotation(np.pi)
        result = matrix33.inverse(m)
        self.assertTrue(np.allclose(result, matrix33.create_from_y_rotation(-np.pi)))

    def test_create_direction_scale(self):
        m = matrix33.create_direction_scale([0.,1.,0.], 0.5)
        v = np.array([
            [1.,0.,0.],
            [1.,1.,1.],
            [10.,10.,10.]
        ])
        result = np.array([
            matrix33.apply_to_vector(m, v[0]),
            matrix33.apply_to_vector(m, v[1]),
            matrix33.apply_to_vector(m, v[2]),
        ])
        expected = np.array([
            [1.,0.,0.],
            [1.,.5,1.],
            [10.,5.,10.]
        ])
        self.assertTrue(np.allclose(result, expected))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""Provide functions for the trigonometric functions.
"""
from __future__ import absolute_import, division, print_function
import math


def aspect_ratio(width, height):
    return float(width) / float(height)

def calculate_fov(zoom, height=1.0):
    """Calculates the required FOV to set the
    view frustrum to have a view with the specified height
    at the specified distance.

    :param float zoom: The distance to calculate the FOV for.
    :param float height: The desired view height at the specified
        distance.
        The default is 1.0.
    :rtype: A float representing the FOV to use in degrees.
    """
    # http://www.glprogramming.com/red/chapter03.html
    rad_theta = 2.0 * math.atan2(height / 2.0, zoom)
    return math.degrees(rad_theta)

def calculate_zoom(fov, height=1.0):
    """Calculates the zoom (distance) from the camera
    with the specified FOV and height of image.

    :param float fov: The FOV to use.
    :param float height: The height of the image at the
        desired distance.
    :rtype: A float representing the zoom (distance) from the camera for the
        desired height at the specified FOV.
    :raise ZeroDivisionError: Raised if the fov is
        0.0.
    """
    return float(height) / math.tan(fov / 2.0)

def calculate_height(fov, zoom):
    """Performs the opposite of calculate_fov.
    Used to find the current height at a specific distance.

    :param float fov: The current FOV.
    :param float zoom: The distance to calculate the height
        for.
    :rtype: A float representing the height at the specified distance for the
        specified FOV.
    """
    height = zoom * (math.tan(fov / 2.0))
    return height

def calculate_plane_size(aspect_ratio, fov, distance):
    """Calculates the width and height of a plane at the
    specified distance using the FOV of the frustrum
    and aspect ratio of the viewport.

    :param float aspect_ratio: The aspect ratio of the viewport.
    :param float fov: The FOV of the frustrum.
    :param float distance: The distance from the origin/camera
        of the plane to calculate.
    :rtype: A tuple of two floats: width and height: The width and height of
        the plane.
    """
    # http://www.songho.ca/opengl/gl_transform.html
    # http://nehe.gamedev.net/article/replacement_for_gluperspective/21002/
    # http://steinsoft.net/index.php?site=Programming/Code%20Snippets/OpenGL/gluperspective&printable=1
    tangent = math.radians(fov)
    height = distance * tangent
    width = height * aspect_ratio

    return width * 2.0, height * 2.0




try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import aabb


class test_aabb(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.aabb
        from pyrr import aabb

    def test_create_zeros(self):
        result = aabb.create_zeros()
        self.assertTrue(np.array_equal(result, [[0,0,0],[0,0,0]]))

    def test_create_from_bounds(self):
        result = aabb.create_from_bounds([-1,-1,-1],[1,1,1])
        self.assertTrue(np.array_equal(result, [[-1,-1,-1],[1,1,1]]))

    def test_create_from_points(self):
        points = np.array([[-1.0,-1.0,-1.0]])
        result = aabb.create_from_points(points)
        expected = np.array([
                [-1.0,-1.0,-1.0],
                [-1.0,-1.0,-1.0]
        ])
        self.assertTrue(np.array_equal(result, expected))

        points = np.array([
            [-1.0,-1.0,-1.0],
            [-1.0, 2.0,-2.0],
        ])
        result = aabb.create_from_points(points)
        expected = np.array([
            [-1.0,-1.0,-2.0],
            [-1.0, 2.0,-1.0],
        ])
        self.assertTrue(np.array_equal(result, expected))

    def test_create_from_aabbs(self):
        # -1
        a1 = np.array([
            [-1.0, 0.0, 0.0 ],
            [-1.0,-1.0,-1.0 ]
        ])
        # +1
        a2 = np.array([
            [ 1.0,-1.0,-1.0 ],
            [ 1.0, 1.0, 1.0 ]
        ])

        # -1 to +1
        result = aabb.create_from_aabbs(np.array([a1, a2]))
        expected = np.array([
            [-1.0,-1.0,-1.0 ],
            [ 1.0, 1.0, 1.0 ]
        ])

        self.assertTrue(np.array_equal(result, expected))

    def test_add_point(self):
        obj = np.array([
            [-1.0,-1.0,-1.0],
            [-1.0,-1.0,-1.0]
        ])
        points = np.array([1.0, 1.0, 1.0])
        result = aabb.add_points(obj, points)
        expected = np.array([
            [-1.0,-1.0,-1.0 ],
            [ 1.0, 1.0, 1.0 ]
        ])
        self.assertTrue(np.array_equal(result, expected))

    def test_add_aabbs(self):
        a = aabb.create_zeros()

        a1 = np.array([
            [-1.0,-1.0,-1.0],
            [ 1.0, 1.0, 1.0]
        ])
        result = aabb.add_aabbs(a, a1)
        expected = np.array([
            [-1.0,-1.0,-1.0],
            [ 1.0, 1.0, 1.0]
        ])
        self.assertTrue(np.array_equal(result, expected))


        a = np.array([
            [-1.0,-1.0,-1.0],
            [ 1.0, 1.0, 1.0]
        ])
        a2 = np.array([
            [-1.0, 0.0,-1.0],
            [ 2.0, 1.0, 1.0]
        ])
        result = aabb.add_aabbs(a, a2)
        expected = np.array([
            [-1.0,-1.0,-1.0],
            [ 2.0, 1.0, 1.0]
        ])
        self.assertTrue(np.array_equal(result, expected))

    def test_centre_point_single_point(self):
        points = np.array([[-1.0,-1.0,-1.0]])
        obj = aabb.create_from_points(points)
        result = aabb.centre_point(obj)
        expected = np.array([-1.0,-1.0,-1.0])
        self.assertTrue(np.array_equal(result, expected))

    def test_centre_point_multiple_points(self):
        points = np.array([
            [ 1.0, 1.0, 1.0],
            [-1.0,-1.0,-1.0]
        ])
        obj = aabb.create_from_points(points)
        result = aabb.centre_point(obj)
        expected = np.zeros(3)
        self.assertTrue(np.array_equal(result, expected))

    def test_minimum(self):
        a = aabb.create_from_bounds([-1,-1,-1],[1,1,1])
        result = aabb.minimum(a)
        self.assertTrue(np.array_equal(result, [-1,-1,-1]))

    def test_maximum(self):
        a = aabb.create_from_bounds([-1,-1,-1],[1,1,1])
        result = aabb.maximum(a)
        self.assertTrue(np.array_equal(result, [1,1,1]))

    def test_clamp_points_single(self):
        a = aabb.create_from_bounds([-1,-1,-1],[1,1,1])
        points = np.array([2,1,1])
        result = aabb.clamp_points(a, points)
        expected = np.array([1,1,1])
        self.assertTrue(np.array_equal(result, expected))

    def test_clamp_points_list(self):
        a = aabb.create_from_bounds([-1,-1,-1],[1,1,1])
        points = np.array([
            [1,1,1],
            [2,1,1],
            [-1,-1,-1],
            [-2,-2,-2],
        ])
        result = aabb.clamp_points(a, points)
        expected = np.array([[1,1,1],[1,1,1],[-1,-1,-1],[-1,-1,-1]])
        self.assertTrue(np.array_equal(result, expected))

if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-
"""4x4 Matrix which supports rotation, translation, scale and skew.

Matrices are laid out in row-major format and can be loaded directly
into OpenGL.
To convert to column-major format, transpose the array using the
numpy.array.T method.
"""
from __future__ import absolute_import, division, print_function
import numpy as np
from . import matrix33
from . import vector
from . import vector3
from . import quaternion
from .utils import all_parameters_as_numpy_arrays, parameters_as_numpy_arrays


def create_identity(dtype=None):
    """Creates a new matrix44 and sets it to
    an identity matrix.

    :rtype: numpy.array
    :return: A matrix representing an identity matrix with shape (4,4).
    """
    return np.identity(4, dtype=dtype)

def create_from_matrix33(mat, dtype=None):
    """Creates a Matrix44 from a Matrix33.

    The translation will be 0,0,0.

    :rtype: numpy.array
    :return: A matrix with shape (4,4) with the input matrix rotation.
    """
    mat4 = np.identity(4, dtype=dtype)
    mat4[0:3, 0:3] = mat
    return mat4

def create_matrix33_view(mat):
    """Returns a view into the matrix in Matrix33 format.

    This is different from matrix33.create_from_matrix44, in that
    changes to the returned matrix will also alter the original matrix.

    :rtype: numpy.array
    :return: A view into the matrix in the format of a matrix33 (shape (3,3)).
    """
    return mat[0:3, 0:3]

@parameters_as_numpy_arrays('eulers')
def create_from_eulers(eulers, dtype=None):
    """Creates a matrix from the specified Euler rotations.

    :param numpy.array eulers: A set of euler rotations in the format
        specified by the euler modules.
    :rtype: numpy.array
    :return: A matrix with shape (4,4) with the euler's rotation.
    """
    dtype = dtype or eulers.dtype
    # set to identity matrix
    # this will populate our extra rows for us
    mat = create_identity(dtype)

    # we'll use Matrix33 for our conversion
    mat[0:3, 0:3] = matrix33.create_from_eulers(eulers, dtype)
    return mat

@parameters_as_numpy_arrays('axis')
def create_from_axis_rotation(axis, theta, dtype=None):
    """Creates a matrix from the specified rotation theta around an axis.

    :param numpy.array axis: A (3,) vector.
    :param float theta: A rotation in radians.

    :rtype: numpy.array
    :return: A matrix with shape (4,4).
    """
    dtype = dtype or axis.dtype
    # set to identity matrix
    # this will populate our extra rows for us
    mat = create_identity(dtype)

    # we'll use Matrix33 for our conversion
    mat[0:3, 0:3] = matrix33.create_from_axis_rotation(axis, theta, dtype)
    return mat

@parameters_as_numpy_arrays('quat')
def create_from_quaternion(quat, dtype=None):
    """Creates a matrix with the same rotation as a quaternion.

    :param quat: The quaternion to create the matrix from.
    :rtype: numpy.array
    :return: A matrix with shape (4,4) with the quaternion's rotation.
    """
    dtype = dtype or quat.dtype
    # set to identity matrix
    # this will populate our extra rows for us
    mat = create_identity(dtype)

    # we'll use Matrix33 for our conversion
    mat[0:3, 0:3] = matrix33.create_from_quaternion(quat, dtype)
    return mat

@parameters_as_numpy_arrays('quat')
def create_from_inverse_of_quaternion(quat, dtype=None):
    """Creates a matrix with the inverse rotation of a quaternion.

    This can be used to go from object space to intertial space.

    :param numpy.array quat: The quaternion to make the matrix from (shape 4).
    :rtype: numpy.array
    :return: A matrix with shape (4,4) that respresents the inverse of
        the quaternion.
    """
    dtype = dtype or quat.dtype
    # set to identity matrix
    # this will populate our extra rows for us
    mat = create_identity(dtype)

    # we'll use Matrix33 for our conversion
    mat[0:3, 0:3] = matrix33.create_from_inverse_of_quaternion(quat, dtype)
    return mat

@parameters_as_numpy_arrays('vec')
def create_from_translation(vec, dtype=None):
    """Creates an identity matrix with the translation set.

    :param numpy.array vec: The translation vector (shape 3 or 4).
    :rtype: numpy.array
    :return: A matrix with shape (4,4) that represents a matrix
        with the translation set to the specified vector.
    """
    dtype = dtype or vec.dtype
    mat = create_identity(dtype)
    mat[3, 0:3] = vec[:3]
    return mat

def create_from_scale(scale, dtype=None):
    """Creates an identity matrix with the scale set.

    :param numpy.array scale: The scale to apply as a vector (shape 3).
    :rtype: numpy.array
    :return: A matrix with shape (4,4) with the scale
        set to the specified vector.
    """
    # we need to expand 'scale' into it's components
    # because numpy isn't flattening them properly.
    m = np.diagflat([scale[0], scale[1], scale[2], 1.0])
    if dtype:
        m = m.astype(dtype)
    return m

def create_from_x_rotation(theta, dtype=None):
    """Creates a matrix with the specified rotation about the X axis.

    :param float theta: The rotation, in radians, about the X-axis.
    :rtype: numpy.array
    :return: A matrix with the shape (4,4) with the specified rotation about
        the X-axis.

    .. seealso:: http://en.wikipedia.org/wiki/Rotation_matrix#In_three_dimensions
    """
    mat = create_identity(dtype)
    mat[0:3, 0:3] = matrix33.create_from_x_rotation(theta, dtype)
    return mat

def create_from_y_rotation(theta, dtype=None):
    """Creates a matrix with the specified rotation about the Y axis.

    :param float theta: The rotation, in radians, about the Y-axis.
    :rtype: numpy.array
    :return: A matrix with the shape (4,4) with the specified rotation about
        the Y-axis.

    .. seealso:: http://en.wikipedia.org/wiki/Rotation_matrix#In_three_dimensions
    """
    mat = create_identity(dtype)
    mat[0:3, 0:3] = matrix33.create_from_y_rotation(theta, dtype)
    return mat

def create_from_z_rotation(theta, dtype=None):
    """Creates a matrix with the specified rotation about the Z axis.

    :param float theta: The rotation, in radians, about the Z-axis.
    :rtype: numpy.array
    :return: A matrix with the shape (4,4) with the specified rotation about
        the Z-axis.

    .. seealso:: http://en.wikipedia.org/wiki/Rotation_matrix#In_three_dimensions
    """
    mat = create_identity(dtype)
    mat[0:3, 0:3] = matrix33.create_from_z_rotation(theta, dtype)
    return mat

@all_parameters_as_numpy_arrays
def apply_to_vector(mat, vec):
    """Apply a matrix to a vector.

    The matrix's rotation and translation are applied to the vector.
    Supports multiple matrices and vectors.

    :param numpy.array mat: The rotation / translation matrix.
        Can be a list of matrices.
    :param numpy.array vec: The vector to modify.
        Can be a list of vectors.
    :rtype: numpy.array
    :return: The vectors rotated by the specified matrix.
    """
    if vec.size == 3:
        # convert to a vec4
        vec4 = np.array([vec[0], vec[1], vec[2], 1.], dtype=vec.dtype)
        vec4 = np.dot(vec4, mat)
        if np.allclose(vec4[3], 0.):
            vec4[:] = [np.inf, np.inf, np.inf, np.inf]
        else:
            vec4 /= vec4[3]
        return vec4[:3]
    elif vec.size == 4:
        return np.dot(vec, mat)
    else:
        raise ValueError("Vector size unsupported")

def multiply(m1, m2):
    """Multiply two matricies, m1 . m2.

    This is essentially a wrapper around
    numpy.dot(m1, m2)

    :param numpy.array m1: The first matrix.
        Can be a list of matrices.
    :param numpy.array m2: The second matrix.
        Can be a list of matrices.
    :rtype: numpy.array
    :return: A matrix that results from multiplying m1 by m2.
    """
    return np.dot(m1, m2)

def create_perspective_projection(fovy, aspect, near, far, dtype=None):
    """Creates perspective projection matrix.

    .. seealso:: http://www.opengl.org/sdk/docs/man2/xhtml/gluPerspective.xml
    .. seealso:: http://www.geeks3d.com/20090729/howto-perspective-projection-matrix-in-opengl/

    :param float fovy: field of view in y direction in degrees
    :param float aspect: aspect ratio of the view (width / height)
    :param float near: distance from the viewer to the near clipping plane (only positive)
    :param float far: distance from the viewer to the far clipping plane (only positive)
    :rtype: numpy.array
    :return: A projection matrix representing the specified perpective.
    """
    ymax = near * np.tan(fovy * np.pi / 360.0)
    xmax = ymax * aspect
    return create_perspective_projection_from_bounds(-xmax, xmax, -ymax, ymax, near, far, dtype=dtype)

def create_perspective_projection_matrix(fovy, aspect, near, far, dtype=None):    # TDOO: mark as deprecated
    """Creates perspective projection matrix.

    .. seealso:: http://www.opengl.org/sdk/docs/man2/xhtml/gluPerspective.xml
    .. seealso:: http://www.geeks3d.com/20090729/howto-perspective-projection-matrix-in-opengl/

    :param float fovy: field of view in y direction in degrees
    :param float aspect: aspect ratio of the view (width / height)
    :param float near: distance from the viewer to the near clipping plane (only positive)
    :param float far: distance from the viewer to the far clipping plane (only positive)
    :rtype: numpy.array
    :return: A projection matrix representing the specified perpective.
    """
    return create_perspective_projection(fovy, aspect, near, far, dtype)

def create_perspective_projection_from_bounds(
    left,
    right,
    bottom,
    top,
    near,
    far,
    dtype=None
):
    """Creates a perspective projection matrix using the specified near
    plane dimensions.

    :param float left: The left of the near plane relative to the plane's centre.
    :param float right: The right of the near plane relative to the plane's centre.
    :param float top: The top of the near plane relative to the plane's centre.
    :param float bottom: The bottom of the near plane relative to the plane's centre.
    :param float near: The distance of the near plane from the camera's origin.
        It is recommended that the near plane is set to 1.0 or above to avoid rendering issues
        at close range.
    :param float far: The distance of the far plane from the camera's origin.
    :rtype: numpy.array
    :return: A projection matrix representing the specified perspective.

    .. seealso:: http://www.gamedev.net/topic/264248-building-a-projection-matrix-without-api/
    .. seealso:: http://www.glprogramming.com/red/chapter03.html
    """

    """
    E 0 A 0
    0 F B 0
    0 0 C D
    0 0-1 0

    A = (right+left)/(right-left)
    B = (top+bottom)/(top-bottom)
    C = -(far+near)/(far-near)
    D = -2*far*near/(far-near)
    E = 2*near/(right-left)
    F = 2*near/(top-bottom)
    """
    A = (right + left) / (right - left)
    B = (top + bottom) / (top - bottom)
    C = -(far + near) / (far - near)
    D = -2. * far * near / (far - near)
    E = 2. * near / (right - left)
    F = 2. * near / (top - bottom)

    return np.array((
        (  E, 0., 0., 0.),
        ( 0.,  F, 0., 0.),
        (  A,  B,  C,-1.),
        ( 0., 0.,  D, 0.),
    ), dtype=dtype)

def create_perspective_projection_matrix_from_bounds(
    left, right, bottom, top, near, far, dtype=None):    # TDOO: mark as deprecated
    """Creates a perspective projection matrix using the specified near
    plane dimensions.

    :param float left: The left of the near plane relative to the plane's centre.
    :param float right: The right of the near plane relative to the plane's centre.
    :param float top: The top of the near plane relative to the plane's centre.
    :param float bottom: The bottom of the near plane relative to the plane's centre.
    :param float near: The distance of the near plane from the camera's origin.
        It is recommended that the near plane is set to 1.0 or above to avoid rendering issues
        at close range.
    :param float far: The distance of the far plane from the camera's origin.
    :rtype: numpy.array
    :return: A projection matrix representing the specified perspective.

    .. seealso:: http://www.gamedev.net/topic/264248-building-a-projection-matrix-without-api/
    .. seealso:: http://www.glprogramming.com/red/chapter03.html
    """

    """
    E 0 A 0
    0 F B 0
    0 0 C D
    0 0-1 0

    A = (right+left)/(right-left)
    B = (top+bottom)/(top-bottom)
    C = -(far+near)/(far-near)
    D = -2*far*near/(far-near)
    E = 2*near/(right-left)
    F = 2*near/(top-bottom)
    """
    return create_perspective_projection_from_bounds(
        left, right, bottom, top, near, far, dtype
    )

def create_orthogonal_projection(
    left,
    right,
    bottom,
    top,
    near,
    far,
    dtype=None
):
    """Creates an orthogonal projection matrix.

    :param float left: The left of the near plane relative to the plane's centre.
    :param float right: The right of the near plane relative to the plane's centre.
    :param float top: The top of the near plane relative to the plane's centre.
    :param float bottom: The bottom of the near plane relative to the plane's centre.
    :param float near: The distance of the near plane from the camera's origin.
        It is recommended that the near plane is set to 1.0 or above to avoid rendering issues
        at close range.
    :param float far: The distance of the far plane from the camera's origin.
    :rtype: numpy.array
    :return: A projection matrix representing the specified orthogonal perspective.

    .. seealso:: http://msdn.microsoft.com/en-us/library/dd373965(v=vs.85).aspx
    """

    """
    A 0 0 Tx
    0 B 0 Ty
    0 0 C Tz
    0 0 0 1

    A = 2 / (right - left)
    B = 2 / (top - bottom)
    C = -2 / (far - near)

    Tx = (right + left) / (right - left)
    Ty = (top + bottom) / (top - bottom)
    Tz = (far + near) / (far - near)
    """
    rml = right - left
    tmb = top - bottom
    fmn = far - near

    A = 2. / rml
    B = 2. / tmb
    C = -2. / fmn
    Tx = -(right + left) / rml
    Ty = -(top + bottom) / tmb
    Tz = -(far + near) / fmn

    return np.array((
        ( A, 0., 0., 0.),
        (0.,  B, 0., 0.),
        (0., 0.,  C, 0.),
        (Tx, Ty, Tz, 1.),
    ), dtype=dtype)

def create_orthogonal_projection_matrix(
    left, right, bottom, top, near, far, dtype=None):    # TDOO: mark as deprecated
    """Creates an orthogonal projection matrix.

    :param float left: The left of the near plane relative to the plane's centre.
    :param float right: The right of the near plane relative to the plane's centre.
    :param float top: The top of the near plane relative to the plane's centre.
    :param float bottom: The bottom of the near plane relative to the plane's centre.
    :param float near: The distance of the near plane from the camera's origin.
        It is recommended that the near plane is set to 1.0 or above to avoid rendering issues
        at close range.
    :param float far: The distance of the far plane from the camera's origin.
    :rtype: numpy.array
    :return: A projection matrix representing the specified orthogonal perspective.

    .. seealso:: http://msdn.microsoft.com/en-us/library/dd373965(v=vs.85).aspx
    """

    """
    A 0 0 Tx
    0 B 0 Ty
    0 0 C Tz
    0 0 0 1

    A = 2 / (right - left)
    B = 2 / (top - bottom)
    C = -2 / (far - near)

    Tx = (right + left) / (right - left)
    Ty = (top + bottom) / (top - bottom)
    Tz = (far + near) / (far - near)
    """
    return create_orthogonal_projection(
        left, right, bottom, top, near, far, dtype
    )

def create_look_at(eye, target, up, dtype=None):
    """Creates a look at matrix according to OpenGL standards.

    :param numpy.array eye: Position of the camera in world coordinates.
    :param numpy.array target: The position in world coordinates that the
        camera is looking at.
    :param numpy.array up: The up vector of the camera.
    :rtype: numpy.array
    :return: A look at matrix that can be used as a viewMatrix
    """

    eye = np.asarray(eye)
    target = np.asarray(target)
    up = np.asarray(up)

    forward = vector.normalize(target - eye)
    side = vector.normalize(np.cross(forward, up))
    up = vector.normalize(np.cross(side, forward))

    return np.array((
            (side[0], up[0], -forward[0], 0.),
            (side[1], up[1], -forward[1], 0.),
            (side[2], up[2], -forward[2], 0.),
            (-np.dot(side, eye), -np.dot(up, eye), np.dot(forward, eye), 1.0)
        ), dtype=dtype)


def inverse(m):
    """Returns the inverse of the matrix.

    This is essentially a wrapper around numpy.linalg.inv.

    :param numpy.array m: A matrix.
    :rtype: numpy.array
    :return: The inverse of the specified matrix.

    .. seealso:: http://docs.scipy.org/doc/numpy/reference/generated/numpy.linalg.inv.html
    """
    return np.linalg.inv(m)


def decompose(m):
    """Decomposes an affine transformation matrix into its scale, rotation and
    translation components.

    :param numpy.array m: A matrix.
    :return: tuple (scale, rotation, translation)
        numpy.array scale vector3
        numpy.array rotation quaternion
        numpy.array translation vector3
    """
    m = np.asarray(m)

    scale = np.linalg.norm(m[:3, :3], axis=1)

    det = np.linalg.det(m)
    if det < 0:
        scale[0] *= -1

    position = m[3, :3]

    rotation = m[:3, :3] * (1 / scale)[:, None]

    return scale, quaternion.create_from_matrix(rotation), position



try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import aambb
from pyrr import aabb
from pyrr import vector


class test_aambb(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.aambb
        from pyrr import aambb

    def test_create_zeros(self):
        result = aambb.create_zeros()
        self.assertTrue(np.array_equal(result, [[0.,0.,0.],[0.,0.,0.]]))
        self.assertTrue(np.array_equal(aambb.centre_point(result), [0.0,0.0,0.0]))

    def test_create_from_bounds(self):
        bounds = [[-1.,1.,-1.], [2.,1.,0.]]
        result = aambb.create_from_bounds(*bounds)
        length = max(vector.length(bounds[0]), vector.length(bounds[1]))
        self.assertTrue(np.array_equal(result, [[-length,-length,-length],[length,length,length]]))
        self.assertTrue(np.array_equal(aambb.centre_point(result), [0.0,0.0,0.0]))

    def test_create_from_points(self):
        result = aambb.create_from_points(np.array([[-1.0, 0.0, 0.0]]))
        self.assertTrue(np.array_equal(result, [[-1.0,-1.0,-1.0],[ 1.0, 1.0, 1.0]]))
        self.assertTrue(np.array_equal(aambb.centre_point(result), [0.0,0.0,0.0]))

    def test_center_point(self):
        # this should always be 0,0,0
        result = aambb.create_from_bounds([-1.,1.,-1.], [2.,1.,0.])
        self.assertTrue(np.array_equal(aambb.centre_point(result), [0.0,0.0,0.0]))

    def test_create_from_aabbs(self):
        a1 = aambb.create_from_points([
            [ 0.0, 0.0, 0.0],
            [ 1.0, 1.0,-1.0]
        ])
        a2 = aambb.create_from_points([
            [ 0.0, 0.0, 2.0],
            [-1.0,-1.0, 1.0]
        ])
        result = aambb.create_from_aabbs([a1, a2])
        length = np.amax(vector.length([a1, a2]))
        self.assertTrue(np.array_equal(result, [[-length,-length,-length],[length,length,length]]), (result,))
        self.assertTrue(np.array_equal(aambb.centre_point(result), [0.0,0.0,0.0]))

    def test_add_point(self):
        a = aambb.create_from_bounds([-0.5,-0.5,-0.5], [0.5,0.5,0.5])
        points = np.array([
            [ 2.0,-1.0,-1.0],
            [ 1.0, 3.0,-1.0],
        ])
        result = aambb.add_points(a, points)
        length = np.amax(vector.length([a, points]))
        self.assertTrue(np.array_equal(result, [[-length,-length,-length],[length,length,length]]), (result,))
        self.assertTrue(np.array_equal(aambb.centre_point(result), [0.0,0.0,0.0]))

    def test_add_aabbs(self):
        a1 = aambb.create_from_bounds([-0.5,-0.5,-0.5], [0.5,0.5,0.5])
        a2 = aambb.create_from_bounds([1.0,-2.0, 1.0], [2.0,-1.0, 1.0])
        result = aambb.add_aabbs(a1, [a2])
        length = np.amax(vector.length([a1, a2]))
        self.assertTrue(np.array_equal(result, [[-length,-length,-length],[length,length,length]]), (result,))
        self.assertTrue(np.array_equal(aambb.centre_point(result), [0.0,0.0,0.0]))


if __name__ == '__main__':
    unittest.main()




try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import matrix44, quaternion, vector3


class test_matrix44(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.matrix44
        from pyrr import matrix44

    def test_create_identity(self):
        result = matrix44.create_identity()
        np.testing.assert_almost_equal(result, np.eye(4), decimal=5)

    def test_create_from_quaternion_unit(self):
        result = matrix44.create_from_quaternion([0.,0.,0.,1.])
        np.testing.assert_almost_equal(result, np.eye(4), decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_quaternion_x(self):
        result = matrix44.create_from_quaternion([1.,0.,0.,0.])
        expected = [
            [1.,0.,0.,0.],
            [0.,-1.,0.,0.],
            [0.,0.,-1.,0.],
            [0.,0.,0.,1.],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_quaternion_y(self):
        result = matrix44.create_from_quaternion([0.,1.,0.,0.])
        expected = [
            [-1.,0.,0.,0.],
            [0.,1.,0.,0.],
            [0.,0.,-1.,0.],
            [0.,0.,0.,1.],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_quaternion_z(self):
        result = matrix44.create_from_quaternion([0.,0.,1.,0.])
        expected = [
            [-1.,0.,0.,0.],
            [0.,-1.,0.,0.],
            [0.,0.,1.,0.],
            [0.,0.,0.,1.],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_quaternion_rotation(self):
        result = matrix44.create_from_quaternion([.57735,.57735,.57735,0.])
        expected = [
            [-0.333333, 0.666667, 0.666667,0.],
            [0.666667, -0.333333, 0.666667,0.],
            [0.666667, 0.666667, -0.333333,0.],
            [0.,0.,0.,1.],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_axis_rotation(self):
        # wolfram alpha can be awesome sometimes
        result = matrix44.create_from_axis_rotation([0.57735, 0.57735, 0.57735],np.pi)
        np.testing.assert_almost_equal(result, matrix44.create_from_quaternion([5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17]), decimal=3)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_axis_rotation_non_normalized(self):
        result = matrix44.create_from_axis_rotation([1.,1.,1.], np.pi)
        np.testing.assert_almost_equal(result, matrix44.create_from_quaternion([5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17]), decimal=3)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_inverse_of_quaternion(self):
        q = quaternion.create_from_x_rotation(np.pi / 2.0)
        result = matrix44.create_from_inverse_of_quaternion(q)
        self.assertTrue(np.allclose(result, matrix44.create_from_x_rotation(-np.pi / 2.0)))

    def test_create_from_translation( self ):
        result = matrix44.create_from_translation([2.,3.,4.])
        expected = np.eye(4)
        expected[3,:3] = [2.,3.,4.]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_create_from_scale( self ):
        result = matrix44.create_from_scale([2.,3.,4.])
        np.testing.assert_almost_equal(result.diagonal()[:-1], [2.,3.,4.], decimal=5)

    def test_create_matrix33_view( self ):
        mat = matrix44.create_identity()
        result = matrix44.create_matrix33_view(mat)
        np.testing.assert_almost_equal(result, mat[:3,:3], decimal=5)
        mat[0,0] = 2.
        np.testing.assert_almost_equal(result, mat[:3,:3], decimal=5)

    def test_create_from_matrix33( self ):
        mat = np.array([
            [1.,2.,3.],
            [3.,4.,5.],
            [6.,7.,8.]
        ])
        result = matrix44.create_from_matrix33(mat)
        np.testing.assert_almost_equal(result[:3,:3], mat, decimal=5)
        orig = mat.copy()
        mat[0,0] = 2.
        np.testing.assert_almost_equal(result[:3,:3], orig, decimal=5)

    def test_create_perspective_projection_matrix_vector3(self):
        def apply_test(m, point, inside):
            p = matrix44.apply_to_vector(m, point)

            # the values are now in clip space from (-1.,-1.,-1.) -> (1.,1.,1.)
            # to be inside = all(-1. < value < 1.)
            self.assertTrue(inside == (np.amax(np.absolute(p)) <= 1.), (inside, point, p))

        m = matrix44.create_perspective_projection_matrix(90, 1024./768., 1., 10.)

        apply_test(m, np.array((0.,0.,0.)), False)
        apply_test(m, np.array((0.,0.,-.5)), False)
        apply_test(m, np.array((0.,0.,-1.)), True)
        apply_test(m, np.array((0.,0.,-2.)), True)
        apply_test(m, np.array((0.,0.,-9.)), True)
        apply_test(m, np.array((0.,0.,-11.)), False)
        apply_test(m, np.array((1.,1.,-5.)), True)

    def test_create_perspective_projection_matrix_dtype(self):
        m1 = matrix44.create_perspective_projection_matrix(90, 1024./768., 1., 10., dtype='float32')
        m2 = matrix44.create_perspective_projection_matrix(90, 1024./768., 1., 10., dtype='float64')
        self.assertEqual(m1.dtype, np.float32)
        self.assertEqual(m2.dtype, np.float64)

    def test_create_perspective_projection_matrix_vector4_inside(self):
        def apply_test(m, point, inside):
            p = matrix44.apply_to_vector(m, point)
            if np.allclose(p[3], 0.):
                self.assertFalse(inside)

            # the values are now in clip space from (-1.,-1.,-1.) -> (1.,1.,1.)
            # to be inside = all(-1. < value < 1.)
            if np.allclose(p[3],0.):
                p[:] = [np.inf,np.inf,np.inf,np.inf]
            else:
                p[:3] /= p[3]
            self.assertTrue(inside == (np.amax(np.absolute(p[:3])) <= 1.), (inside, point, p))

        m = matrix44.create_perspective_projection_matrix(90, 1024./768., 1., 10.)
        apply_test(m, np.array((0.,0.,0.,1.)), False)
        apply_test(m, np.array((0.,0.,-.5,1.)), False)
        apply_test(m, np.array((0.,0.,-1.,1.)), True)
        apply_test(m, np.array((0.,0.,-2.,1.)), True)
        apply_test(m, np.array((0.,0.,-9.,1.)), True)
        apply_test(m, np.array((0.,0.,-11.,1.)), False)
        apply_test(m, np.array((1.,1.,-5.,1.)), True)

    def test_create_orthogonal_projection_matrix_vector3(self):
        def apply_test(m, point, inside):
            p = matrix44.apply_to_vector(m, point)

            # the values are now in clip space from (-1.,-1.,-1.) -> (1.,1.,1.)
            # to be inside = all(-1. < value < 1.)
            self.assertTrue(inside == (np.amax(np.absolute(p[:3])) <= 1.), (inside, point, p))

        m = matrix44.create_orthogonal_projection_matrix(-1., 1., -1., 1., 1., 10.)

        # +Z
        apply_test(m, np.array((0.,0.,0.)), False)
        apply_test(m, np.array((0.,0.,1.)), False)
        # -Z but outside near, far
        apply_test(m, np.array((0.,0.,-.5)), False)
        apply_test(m, np.array((0.,0.,-11.)), False)
        apply_test(m, np.array((0.,0.,1.)), False)
        # Valid
        apply_test(m, np.array((0.,0.,-10.)), True)
        apply_test(m, np.array((0.,0.,-1.)), True)
        apply_test(m, np.array((0.,0.,-2.)), True)
        apply_test(m, np.array((0.,0.,-9.)), True)
        apply_test(m, np.array((-1.,-1.,-1.)), True)
        apply_test(m, np.array((-1.,-1.,-10.)), True)
        apply_test(m, np.array((1.,1.,-1.)), True)
        apply_test(m, np.array((1.,1.,-10.)), True)
        # Outside left, right, top, bottom
        apply_test(m, np.array((1.1,1.1,-1.)), False)
        apply_test(m, np.array((-1.1,-1.1,-1.)), False)
        apply_test(m, np.array((1.1,1.1,-10.)), False)
        apply_test(m, np.array((-1.1,-1.1,-10.)), False)


    def test_create_orthogonal_projection_matrix_vector4(self):
        def apply_test(m, point, inside):
            p = matrix44.apply_to_vector(m, point)
            if p[3] == 0.:
                self.assertFalse(inside)

            # the values are now in clip space from (-1.,-1.,-1.) -> (1.,1.,1.)
            # to be inside = all(-1. < value < 1.)
            self.assertTrue(inside == (np.amax(np.absolute(p[:3])) <= 1.), (inside, point, p))

        m = matrix44.create_orthogonal_projection_matrix(-1., 1., -1., 1., 1., 10.)

        # +Z
        apply_test(m, np.array((0.,0.,0.,1.)), False)
        apply_test(m, np.array((0.,0.,1.,1.)), False)
        # -Z but outside near, far
        apply_test(m, np.array((0.,0.,-.5,1.)), False)
        apply_test(m, np.array((0.,0.,-11.,1.)), False)
        apply_test(m, np.array((0.,0.,1.,1.)), False)
        # Valid
        apply_test(m, np.array((0.,0.,-10.,1.)), True)
        apply_test(m, np.array((0.,0.,-1.,1.)), True)
        apply_test(m, np.array((0.,0.,-2.,1.)), True)
        apply_test(m, np.array((0.,0.,-9.,1.)), True)
        apply_test(m, np.array((-1.,-1.,-1.,1.)), True)
        apply_test(m, np.array((-1.,-1.,-10.,1.)), True)
        apply_test(m, np.array((1.,1.,-1.,1.)), True)
        apply_test(m, np.array((1.,1.,-10.,1.)), True)
        # Outside left, right, top, bottom
        apply_test(m, np.array((1.1,1.1,-1.,1.)), False)
        apply_test(m, np.array((-1.1,-1.1,-1.,1.)), False)
        apply_test(m, np.array((1.1,1.1,-10.,1.)), False)
        apply_test(m, np.array((-1.1,-1.1,-10.,1.)), False)

    def create_perspective_projection_matrix_from_bounds_vector3(self):
        def apply_test(m, point, inside):
            p = matrix44.apply_to_vector(m, point)

            # the values are now in clip space from (-1.,-1.,-1.) -> (1.,1.,1.)
            # to be inside = all(-1. < value < 1.)
            self.assertTrue(inside == (np.amax(np.absolute(p[:3])) <= 1.), (inside, point, p))

        m = matrix44.create_perspective_projection_matrix_from_bounds(-1.,1.,-1.,1.,1.,10.)

        # +Z
        apply_test(m, np.array((0.,0.,0.)), False)
        apply_test(m, np.array((0.,0.,1.)), False)
        # -Z but outside near, far
        apply_test(m, np.array((0.,0.,-.5)), False)
        apply_test(m, np.array((0.,0.,-11.)), False)
        apply_test(m, np.array((0.,0.,1.)), False)
        # Valid
        apply_test(m, np.array((0.,0.,-10.)), True)
        apply_test(m, np.array((0.,0.,-1.)), True)
        apply_test(m, np.array((0.,0.,-2.)), True)
        apply_test(m, np.array((0.,0.,-9.)), True)
        apply_test(m, np.array((-1.,-1.,-1.)), True)
        apply_test(m, np.array((-1.,-1.,-10.)), True)
        apply_test(m, np.array((1.,1.,-1.)), True)
        apply_test(m, np.array((1.,1.,-10.)), True)
        # Outside left, right, top, bottom
        apply_test(m, np.array((1.1,1.1,-1.)), False)
        apply_test(m, np.array((-1.1,-1.1,-1.)), False)
        apply_test(m, np.array((1.1,1.1,-10.)), False)
        apply_test(m, np.array((-1.1,-1.1,-10.)), False)

    def create_perspective_projection_matrix_from_bounds_vector4(self):
        def apply_test(m, point, inside):
            p = matrix44.apply_to_vector(m, point)
            if p[3] == 0.:
                self.assertFalse(inside)

            # the values are now in clip space from (-1.,-1.,-1.) -> (1.,1.,1.)
            # to be inside = all(-1. < value < 1.)
            self.assertTrue(inside == (np.amax(np.absolute(p[:3])) <= 1.), (inside, point, p))

        m = matrix44.create_perspective_projection_matrix_from_bounds(-1.,1.,-1.,1.,1.,10.)

        # +Z
        apply_test(m, np.array((0.,0.,0.,1.)), False)
        apply_test(m, np.array((0.,0.,1.,1.)), False)
        # -Z but outside near, far
        apply_test(m, np.array((0.,0.,-.5,1.)), False)
        apply_test(m, np.array((0.,0.,-11.,1.)), False)
        apply_test(m, np.array((0.,0.,1.,1.)), False)
        # Valid
        apply_test(m, np.array((0.,0.,-10.,1.)), True)
        apply_test(m, np.array((0.,0.,-1.,1.)), True)
        apply_test(m, np.array((0.,0.,-2.,1.)), True)
        apply_test(m, np.array((0.,0.,-9.,1.)), True)
        apply_test(m, np.array((-1.,-1.,-1.,1.)), True)
        apply_test(m, np.array((-1.,-1.,-10.,1.)), True)
        apply_test(m, np.array((1.,1.,-1.,1.)), True)
        apply_test(m, np.array((1.,1.,-10.,1.)), True)
        # Outside left, right, top, bottom
        apply_test(m, np.array((1.1,1.1,-1.,1.)), False)
        apply_test(m, np.array((-1.1,-1.1,-1.,1.)), False)
        apply_test(m, np.array((1.1,1.1,-10.,1.)), False)
        apply_test(m, np.array((-1.1,-1.1,-10.,1.)), False)

    def test_create_look_at_determinant(self):
        m = matrix44.create_look_at(
            np.array((300.0, 200.0, 100.0)),
            np.array((0.0, 0.0, 0.0)),
            np.array((0.0, 0.0, 1.0)),
        )

        self.assertAlmostEqual(np.linalg.det(m), 1.0)

    def test_create_look_at(self):
        m = matrix44.create_look_at(
            np.array((300.0, 200.0, 100.0)),
            np.array((0.0, 0.0, 10.0)),
            np.array((0.0, 0.0, 1.0)),
        )

        points = [
            (-10.0, -10.0, 0.0, 1.0),
            (-10.0, 10.0, 0.0, 1.0),
            (10.0, -10.0, 0.0, 1.0),
            (10.0, 10.0, 0.0, 1.0),
            (-10.0, -10.0, 20.0, 1.0),
            (-10.0, 10.0, 20.0, 1.0),
            (10.0, -10.0, 20.0, 1.0),
            (10.0, 10.0, 20.0, 1.0),
        ]

        for point in points:
            x, y, z, w = matrix44.apply_to_vector(m, point)
            self.assertTrue(-20.0 < x and x < 20.0)
            self.assertTrue(-20.0 < y and y < 20.0)
            self.assertTrue(z < 0.0)
            self.assertAlmostEqual(w, 1.0)

    def test_create_look_at_2(self):
        m = matrix44.create_look_at(
            np.array((10.0, 0.0, 0.0)),
            np.array((-10.0, 0.0, 0.0)),
            np.array((0.0, 1.0, 0.0)),
        )

        x, y, z, _ = matrix44.apply_to_vector(m, (1.0, 0.0, 0.0, 1.0))
        self.assertAlmostEqual(x, 0.0)
        self.assertAlmostEqual(y, 0.0)
        self.assertAlmostEqual(z, -9.0)

        x, y, z, _ = matrix44.apply_to_vector(m, (0.0, 1.0, 0.0, 1.0))
        self.assertAlmostEqual(x, 0.0)
        self.assertAlmostEqual(y, 1.0)
        self.assertAlmostEqual(z, -10.0)

        x, y, z, _ = matrix44.apply_to_vector(m, (0.0, 0.0, 1.0, 1.0))
        self.assertAlmostEqual(x, -1.0)
        self.assertAlmostEqual(y, 0.0)
        self.assertAlmostEqual(z, -10.0)

    def test_create_look_at_3(self):
        m = matrix44.create_look_at(
            np.array((10.0, 0.0, 0.0)),
            np.array((-10.0, 0.0, 0.0)),
            np.array((0.0, 1.0, 0.0)),
        )

        x, y, z, _ = matrix44.apply_to_vector(m, (1.0, 0.0, 0.0, 0.0))
        self.assertAlmostEqual(x, 0.0)
        self.assertAlmostEqual(y, 0.0)
        self.assertAlmostEqual(z, 1.0)

        x, y, z, _ = matrix44.apply_to_vector(m, (0.0, 1.0, 0.0, 0.0))
        self.assertAlmostEqual(x, 0.0)
        self.assertAlmostEqual(y, 1.0)
        self.assertAlmostEqual(z, 0.0)

        x, y, z, _ = matrix44.apply_to_vector(m, (0.0, 0.0, 1.0, 0.0))
        self.assertAlmostEqual(x, -1.0)
        self.assertAlmostEqual(y, 0.0)
        self.assertAlmostEqual(z, 0.0)

    def test_create_look_at_4(self):
        m = matrix44.create_look_at(
            np.array((0.0, 0.0, 0.0)),
            np.array((0.0, 0.0, -1.0)),
            np.array((0.0, 1.0, 0.0)),
        )

        x, y, z, _ = matrix44.apply_to_vector(m, (1.0, 0.0, 0.0, 0.0))
        self.assertAlmostEqual(x, 1.0)
        self.assertAlmostEqual(y, 0.0)
        self.assertAlmostEqual(z, 0.0)

        x, y, z, _ = matrix44.apply_to_vector(m, (0.0, 1.0, 0.0, 0.0))
        self.assertAlmostEqual(x, 0.0)
        self.assertAlmostEqual(y, 1.0)
        self.assertAlmostEqual(z, 0.0)

        x, y, z, _ = matrix44.apply_to_vector(m, (0.0, 0.0, 1.0, 0.0))
        self.assertAlmostEqual(x, 0.0)
        self.assertAlmostEqual(y, 0.0)
        self.assertAlmostEqual(z, 1.0)

    def test_apply_to_vector_identity(self):
        mat = matrix44.create_identity()
        result = matrix44.apply_to_vector(mat, [1.,0.,0.])
        np.testing.assert_almost_equal(result, [1.,0.,0.], decimal=5)

    def test_apply_to_vector_x_rotation(self):
        mat = matrix44.create_from_x_rotation(np.pi)
        result = matrix44.apply_to_vector(mat, [0.,1.,0.])
        np.testing.assert_almost_equal(result, [0.,-1.,0.], decimal=5)

    def test_apply_to_vector_y_rotation(self):
        mat = matrix44.create_from_y_rotation(np.pi)
        result = matrix44.apply_to_vector(mat, [1.,0.,0.])
        np.testing.assert_almost_equal(result, [-1.,0.,0.], decimal=5)

    def test_apply_to_vector_z_rotation(self):
        mat = matrix44.create_from_z_rotation(np.pi)
        result = matrix44.apply_to_vector(mat, [1.,0.,0.])
        np.testing.assert_almost_equal(result, [-1.,0.,0.], decimal=5)

    def test_apply_to_vector_with_translation(self):
        mat = matrix44.create_from_translation([2.,3.,4.])
        result = matrix44.apply_to_vector(mat, [1.,1.,1.])
        np.testing.assert_almost_equal(result, [3.,4.,5.], decimal=5)

    @unittest.skip('Not implemented')
    def test_create_from_eulers(self):
        # just call the function
        # TODO: check the result
        matrix44.create_from_eulers([1,2,3])

    def test_create_from_x_rotation(self):
        mat = matrix44.create_from_x_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(np.dot([1.,0.,0.,1.], mat), [1.,0.,0.,1.]))
        self.assertTrue(np.allclose(np.dot([0.,1.,0.,1.], mat), [0.,0.,-1.,1.]))
        self.assertTrue(np.allclose(np.dot([0.,0.,1.,1.], mat), [0.,1.,0.,1.]))

    def test_create_from_y_rotation(self):
        mat = matrix44.create_from_y_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(np.dot([1.,0.,0.,1.], mat), [0.,0.,1.,1.]))
        self.assertTrue(np.allclose(np.dot([0.,1.,0.,1.], mat), [0.,1.,0.,1.]))
        self.assertTrue(np.allclose(np.dot([0.,0.,1.,1.], mat), [-1.,0.,0.,1.]))

    def test_create_from_z_rotation(self):
        mat = matrix44.create_from_z_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(np.dot([1.,0.,0.,1.], mat), [0.,-1.,0.,1.]))
        self.assertTrue(np.allclose(np.dot([0.,1.,0.,1.], mat), [1.,0.,0.,1.]))
        self.assertTrue(np.allclose(np.dot([0.,0.,1.,1.], mat), [0.,0.,1.,1.]))

    def test_multiply_identity(self):
        m1 = matrix44.create_identity()
        m2 = matrix44.create_identity()
        result = matrix44.multiply(m1, m2)
        self.assertTrue(np.allclose(result, np.dot(m1,m2)))

    def test_multiply_rotation(self):
        m1 = matrix44.create_from_x_rotation(np.pi)
        m2 = matrix44.create_from_y_rotation(np.pi / 2.0)
        result = matrix44.multiply(m1, m2)
        self.assertTrue(np.allclose(result, np.dot(m1,m2)))

    def test_inverse(self):
        m = matrix44.create_from_y_rotation(np.pi)
        result = matrix44.inverse(m)
        self.assertTrue(np.allclose(result, matrix44.create_from_y_rotation(-np.pi)))
    
    def test_decompose(self):
        # define expectations
        expected_scale = vector3.create(*[1, 1, 2], dtype='f4')
        expected_rotation = quaternion.create_from_y_rotation(np.pi, dtype='f4')
        expected_translation = vector3.create(*[10, 0, -5], dtype='f4')
        expected_model = np.array([
            [-1, 0, 0, 0],
            [0, 1, 0, 0],
            [0, 0, -2, 0],
            [10, 0, -5, 1],
        ], dtype='f4')

        # compose matrix using Pyrr
        s = matrix44.create_from_scale(expected_scale, dtype='f4')
        r = matrix44.create_from_quaternion(expected_rotation, dtype='f4')
        t = matrix44.create_from_translation(expected_translation, dtype='f4')
        model = s.dot(r).dot(t)
        np.testing.assert_almost_equal(model, expected_model)
        self.assertTrue(model.dtype == expected_model.dtype)

        # decompose matrix
        scale, rotation, translation = matrix44.decompose(model)
        np.testing.assert_almost_equal(scale, expected_scale)
        self.assertTrue(scale.dtype == expected_scale.dtype)
        np.testing.assert_almost_equal(rotation, expected_rotation)
        self.assertTrue(rotation.dtype == expected_rotation.dtype)
        np.testing.assert_almost_equal(translation, expected_translation)
        self.assertTrue(translation.dtype == expected_translation.dtype)


if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import plane, vector

class test_plane(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.plane
        from pyrr import plane

    def test_create(self):
        result = plane.create()
        self.assertTrue(np.allclose(result, [0,0,1,0]))

        result = plane.create([1.,0.,0.], 5.)
        self.assertTrue(np.allclose(result, [1.,0.,0.,5.]))

    def test_create_from_points(self):
        result = plane.create_from_points(
            [1., 0., 0.],
            [0., 1., 0.],
            [1., 1., 0.],
        )
        self.assertTrue(np.allclose(result, [0.,0.,1.,0.]))
        self.assertTrue(np.allclose(plane.position(result), [0., 0., 0.]))

        result = plane.create_from_points(
            [1., 1., 0.],
            [1., 1., 1.],
            [0., 1., 1.],
        )
        expected = plane.create([0.,1.,0.], 1.)
        self.assertTrue(np.allclose(result, expected))
        self.assertTrue(np.allclose(plane.position(result), [0., 1., 0.]))

    def test_create_from_position(self):
        position = np.array([1.0, 0.0, 0.0])
        normal = np.array([0.0, 3.0, 0.0])
        result = plane.create_from_position(position, normal)
        self.assertTrue(np.allclose(result, [0., 1., 0., 0.]))

        p0 = position + [1., 0., 0.]
        p = position
        n = vector.normalise(normal)
        coplanar = p - p0
        self.assertEqual(np.sum(n * coplanar), 0.)

    def test_create_xy(self):
        result = plane.create_xy()
        self.assertTrue(np.allclose(result, [0., 0., 1., 0.]))

        result = plane.create_xy(distance=2.)
        self.assertTrue(np.allclose(result, [0., 0., 1., 2.]))

        result = plane.create_xy(invert=True, distance=2.)
        self.assertTrue(np.allclose(result, [0., 0., -1., -2.]))

    def test_create_xz(self):
        result = plane.create_xz()
        self.assertTrue(np.allclose(result, [0., 1., 0., 0.]))

        result = plane.create_xz(distance=2.)
        self.assertTrue(np.allclose(result, [0., 1., 0., 2.]))

        result = plane.create_xz(invert=True, distance=2.)
        self.assertTrue(np.allclose(result, [0., -1., 0., -2.]))

    def test_create_yz(self):
        result = plane.create_yz()
        self.assertTrue(np.allclose(result, [1., 0., 0., 0.]))

        result = plane.create_yz(distance=2.)
        self.assertTrue(np.allclose(result, [1., 0., 0., 2.]))

        result = plane.create_yz(invert=True, distance=2.)
        print(result)
        self.assertTrue(np.allclose(result, [-1., 0., 0., -2.]))

    def test_invert_normal(self):
        p = np.array([1.0, 0.0, 0.0, 1.0])
        result = plane.invert_normal(p)
        self.assertTrue(np.allclose(result, [-1.0, 0.0, 0.0, -1.0]))

    def test_position(self):
        p = plane.create_xz(distance=-5.)
        result = plane.position(p)
        self.assertTrue(np.allclose(result, [0.,-5.,0.]))

        p = plane.create_from_position(position=[0., 0., 1.], normal=[0., 0., 1.])
        self.assertTrue(np.allclose(plane.position(p), [0., 0., 1.]))

    def test_normal(self):
        p = plane.create_xz(distance=5.)
        result = plane.normal(p)
        self.assertTrue(np.allclose(result, [0.,1.,0.]))

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

# get our version but don't import it
# or we'll need our dependencies already installed
# https://github.com/todddeluca/happybase/commit/63573cdaefe3a2b98ece87e19d9ceb18f00bc0d9
with open('pyrr/version.py', 'r') as f:
    exec(f.read())

setup(
    name='pyrr',
    version=__version__,
    description='3D mathematical functions using NumPy',
    license='BSD',
    author='Adam Griffiths',
    url='https://github.com/adamlwgriffiths/Pyrr',
    install_requires=['numpy', 'multipledispatch'],
    platforms=['any'],
    test_suite='tests',
    packages=['pyrr', 'pyrr.objects'],
    classifiers=[
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'License :: OSI Approved :: BSD License',
        'Topic :: Multimedia :: Graphics',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



from __future__ import absolute_import
try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr.objects.matrix33 import Matrix33
from pyrr.objects.matrix44 import Matrix44
from pyrr.objects.quaternion import Quaternion
from pyrr.objects.vector3 import Vector3
from pyrr.objects.vector4 import Vector4
from pyrr import matrix33
from pyrr import matrix44
from pyrr import quaternion


class test_object_matrix33(unittest.TestCase):
    _shape = (3,3)
    _size = np.multiply.reduce(_shape)

    def test_imports(self):
        import pyrr
        pyrr.Matrix33()
        pyrr.matrix33.Matrix33()
        pyrr.objects.matrix33.Matrix33()

        from pyrr import Matrix33
        from pyrr.objects import Matrix33
        from pyrr.objects.matrix33 import Matrix33

    def test_create(self):
        m = Matrix33()
        self.assertTrue(np.array_equal(m, np.zeros(self._shape)))
        self.assertEqual(m.shape, self._shape)

        m = Matrix33(np.arange(self._size))
        self.assertEqual(m.shape, self._shape)

        m = Matrix33([[1,2,3],[4,5,6],[7,8,9]])
        self.assertEqual(m.shape, self._shape)

        m = Matrix33(Matrix33())
        self.assertTrue(np.array_equal(m, np.zeros(self._shape)))
        self.assertEqual(m.shape, self._shape)

    def test_identity(self):
        m = Matrix33.identity()
        self.assertTrue(np.array_equal(m, np.eye(3)))

    @unittest.skip('Not implemented')
    def test_perspective_projection(self):
        pass

    @unittest.skip('Not implemented')
    def test_perspective_projection_bounds(self):
        pass

    @unittest.skip('Not implemented')
    def test_orthogonal_projection(self):
        pass

    @unittest.skip('Not implemented')
    def test_from_translation(self):
        pass

    def test_create_from_matrix44(self):
        m1 = Matrix44.identity()
        m = Matrix33.from_matrix44(m1)
        self.assertTrue(np.array_equal(m, np.eye(3)))

        m = Matrix33(m1)
        self.assertTrue(np.array_equal(m, np.eye(3)))

    def test_create_from_scale(self):
        v = Vector3([1,2,3])
        m = Matrix33.from_scale(v)
        self.assertTrue(np.array_equal(m, np.diag([1,2,3])))

    def test_create_from_eulers(self):
        e = Vector3([1,2,3])
        m = Matrix33.from_eulers(e)
        self.assertTrue(np.array_equal(m, matrix33.create_from_eulers([1,2,3])))

    def test_create_from_quaternion(self):
        q = Quaternion()
        m = Matrix33.from_quaternion(q)
        self.assertTrue(np.array_equal(m, np.eye(3)))
        self.assertTrue(np.array_equal(m.quaternion, q))

        m = Matrix33(q)
        self.assertTrue(np.array_equal(m, np.eye(3)))

    def test_create_from_inverse_quaternion(self):
        q = Quaternion.from_x_rotation(0.5)
        m = Matrix33.from_inverse_of_quaternion(q)
        expected = matrix33.create_from_quaternion(quaternion.inverse(quaternion.create_from_x_rotation(0.5)))
        np.testing.assert_almost_equal(np.array(m), expected, decimal=5)
        #self.assertTrue(np.array_equal(m, expected))

    def test_multiply(self):
        m1 = Matrix33(np.arange(self._size))
        m2 = Matrix33(np.arange(self._size)[::-1])
        m = m1 * m2
        self.assertTrue(np.array_equal(m, matrix33.multiply(m2, m1)))

        m1 = Matrix33(np.arange(self._size))
        m2 = Matrix44(np.arange(16))
        m = m1 * m2
        self.assertTrue(np.array_equal(m, matrix33.multiply(matrix33.create_from_matrix44(m2), m1)))

    def test_inverse(self):
        m1 = Matrix33.identity() * Matrix33.from_x_rotation(0.5)
        m = m1.inverse
        self.assertTrue(np.array_equal(m, matrix33.inverse(m1)))

    def test_matrix33(self):
        m1 = Matrix33.identity() * Matrix33.from_x_rotation(0.5)
        m = m1.matrix33
        self.assertTrue(m1 is m)

    def test_matrix44(self):
        m1 = Matrix33.identity() * Matrix33.from_x_rotation(0.5)
        m = m1.matrix44
        self.assertTrue(np.array_equal(m, matrix44.create_from_matrix33(m1)))

    def test_operators_matrix33(self):
        m1 = Matrix33.identity()
        m2 = Matrix33.from_x_rotation(0.5)

        # add
        self.assertTrue(np.array_equal(m1 + m2, matrix33.create_identity() + matrix33.create_from_x_rotation(0.5)))

        # subtract
        self.assertTrue(np.array_equal(m1 - m2, matrix33.create_identity() - matrix33.create_from_x_rotation(0.5)))

        # multiply
        self.assertTrue(np.array_equal(m1 * m2, matrix33.multiply(matrix33.create_from_x_rotation(0.5), matrix33.create_identity())))

        # divide
        self.assertRaises(ValueError, lambda: m1 / m2)

        # inverse
        self.assertTrue(np.array_equal(~m2, matrix33.inverse(matrix33.create_from_x_rotation(0.5))))

        # ==
        self.assertTrue(Matrix33() == Matrix33())
        self.assertFalse(Matrix33() == Matrix33([1. for n in range(9)]))

        # !=
        self.assertTrue(Matrix33() != Matrix33([1. for n in range(9)]))
        self.assertFalse(Matrix33() != Matrix33())

    def test_operators_matrix44(self):
        m1 = Matrix33.identity()
        m2 = Matrix44.from_x_rotation(0.5)

        # add
        self.assertTrue(np.array_equal(m1 + m2, matrix33.create_identity() + matrix33.create_from_x_rotation(0.5)))

        # subtract
        self.assertTrue(np.array_equal(m1 - m2, matrix33.create_identity() - matrix33.create_from_x_rotation(0.5)))

        # multiply
        self.assertTrue(np.array_equal(m1 * m2, matrix33.multiply(matrix33.create_from_x_rotation(0.5), matrix33.create_identity())))

        # divide
        self.assertRaises(ValueError, lambda: m1 / m2)

    def test_operators_quaternion(self):
        m = Matrix33.identity()
        q = Quaternion.from_x_rotation(0.7)

        # add
        self.assertRaises(ValueError, lambda: m + q)

        # subtract
        self.assertRaises(ValueError, lambda: m - q)

        # multiply
        self.assertTrue(np.array_equal(m * q, matrix33.multiply(matrix33.create_from_quaternion(quaternion.create_from_x_rotation(0.7)), matrix33.create_identity())))

        # divide
        self.assertRaises(ValueError, lambda: m / q)

    def test_operators_vector3(self):
        m = Matrix33.identity()
        v = Vector3([1,1,1])

        # add
        self.assertRaises(ValueError, lambda: m + v)

        # subtract
        self.assertRaises(ValueError, lambda: m - v)

        # multiply
        self.assertTrue(np.array_equal(m * v, matrix33.apply_to_vector(matrix33.create_identity(), [1,1,1])))

        # divide
        self.assertRaises(ValueError, lambda: m / v)

    def test_operators_vector4(self):
        m = Matrix33.identity()
        v = Vector4([1,1,1,1])

        # add
        self.assertRaises(ValueError, lambda: m + v)

        # subtract
        self.assertRaises(ValueError, lambda: m - v)

        # multiply
        self.assertTrue(ValueError, lambda: m * v)

        # divide
        self.assertRaises(ValueError, lambda: m / v)

    def test_operators_number(self):
        m = Matrix33.identity()
        fv = np.empty((1,), dtype=[('i', np.int16, 1),('f', np.float32, 1)])
        fv[0] = (2, 2.0)

        # add
        self.assertTrue(np.array_equal(m + 1.0, matrix33.create_identity()[:] + 1.0))
        self.assertTrue(np.array_equal(m + 1, matrix33.create_identity()[:] + 1.0))
        self.assertTrue(np.array_equal(m + np.float(1.), matrix33.create_identity()[:] + 1.0))
        self.assertTrue(np.array_equal(m + fv[0]['f'], matrix33.create_identity()[:] + 2.0))
        self.assertTrue(np.array_equal(m + fv[0]['i'], matrix33.create_identity()[:] + 2.0))

        # subtract
        self.assertTrue(np.array_equal(m - 1.0, matrix33.create_identity()[:] - 1.0))
        self.assertTrue(np.array_equal(m - 1, matrix33.create_identity()[:] - 1.0))
        self.assertTrue(np.array_equal(m - np.float(1.), matrix33.create_identity()[:] - 1.0))
        self.assertTrue(np.array_equal(m - fv[0]['f'], matrix33.create_identity()[:] - 2.0))
        self.assertTrue(np.array_equal(m - fv[0]['i'], matrix33.create_identity()[:] - 2.0))

        # multiply
        self.assertTrue(np.array_equal(m * 2.0, matrix33.create_identity()[:] * 2.0))
        self.assertTrue(np.array_equal(m * 2, matrix33.create_identity()[:] * 2.0))
        self.assertTrue(np.array_equal(m * np.float(2.), matrix33.create_identity()[:] * 2.0))
        self.assertTrue(np.array_equal(m * fv[0]['f'], matrix33.create_identity()[:] * 2.0))
        self.assertTrue(np.array_equal(m * fv[0]['i'], matrix33.create_identity()[:] * 2.0))

        # divide
        self.assertTrue(np.array_equal(m / 2.0, matrix33.create_identity()[:] / 2.0))
        self.assertTrue(np.array_equal(m / 2, matrix33.create_identity()[:] / 2.0))
        self.assertTrue(np.array_equal(m / np.float(2.), matrix33.create_identity()[:] / 2.0))
        self.assertTrue(np.array_equal(m / fv[0]['f'], matrix33.create_identity()[:] / 2.0))
        self.assertTrue(np.array_equal(m / fv[0]['i'], matrix33.create_identity()[:] / 2.0))

    def test_accessors(self):
        m = Matrix33(np.arange(self._size))
        self.assertTrue(np.array_equal(m.m1,[0,1,2]))
        self.assertTrue(np.array_equal(m.m2,[3,4,5]))
        self.assertTrue(np.array_equal(m.m3,[6,7,8]))

        self.assertTrue(np.array_equal(m.r1,[0,1,2]))
        self.assertTrue(np.array_equal(m.r2,[3,4,5]))
        self.assertTrue(np.array_equal(m.r3,[6,7,8]))

        self.assertTrue(np.array_equal(m.c1,[0,3,6]))
        self.assertTrue(np.array_equal(m.c2,[1,4,7]))
        self.assertTrue(np.array_equal(m.c3,[2,5,8]))

        self.assertEqual(m.m11, 0)
        self.assertEqual(m.m12, 1)
        self.assertEqual(m.m13, 2)
        self.assertEqual(m.m21, 3)
        self.assertEqual(m.m22, 4)
        self.assertEqual(m.m23, 5)
        self.assertEqual(m.m31, 6)
        self.assertEqual(m.m32, 7)
        self.assertEqual(m.m33, 8)

        m.m11 = 1
        self.assertEqual(m.m11, 1)
        self.assertEqual(m[0,0], 1)
        m.m11 += 1
        self.assertEqual(m.m11, 2)
        self.assertEqual(m[0,0], 2)

if __name__ == '__main__':
    unittest.main()
